@@ -256,7 +256,7 @@ brlconf_setting_new (void)
     new_braille = ( Braille * ) g_new0 ( Braille , 1);
     
     if (!new_braille)	
-	sru_error (_("Unable to allocate memory"));
+	sru_error (_("Unable to allocate memory."));
     
     return new_braille ;
 }
@@ -360,8 +360,7 @@ brlconf_style_set (const gchar *style)
 {
     if (!gnopiconf_set_string (style, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_STYLE))
     {
-	sru_warning (_("brlconf:Failed to set Braille Style:%s\n"), style);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set braille style: %s."), style);
     }
 }
 
@@ -370,8 +369,7 @@ brlconf_cursor_style_set (const gchar *cursor)
 {
     if (!gnopiconf_set_string (cursor, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_CURSOR_STYLE))
     {
-        sru_warning (_("brlconf:Failed to set Braille Cursor Style:%s\n"), cursor);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set braille cursor style: %s."), cursor);
     }
 }
 
@@ -380,8 +378,7 @@ brlconf_attributes_set (gint dots)
 {
     if (!gnopiconf_set_int (dots, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_ATTRIBUTES))
     {
-	sru_warning (_("brlconf:Failed to set Atrribute settings:%d\n"), dots);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set attribute settings: %d."), dots);
     }
 }
 
@@ -390,8 +387,7 @@ brlconf_device_set (const gchar *device)
 {
     if (!gnopiconf_set_string (device, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_DEVICE))
     {
-        sru_warning (_("brlconf:Failed to set Braille Device:%s\n"), device);
-        sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set braille device: %s."), device);
     }
 }
 
@@ -400,8 +396,7 @@ brlconf_fill_char_set (const gchar *texture)
 {
     if (!gnopiconf_set_string (texture, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_FILL_CHAR))
     {
-        sru_warning (_("brlconf:Failed to set Fill Char:%s\n"), texture);
-        sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set fill character: %s."), texture);
     }
 }
 
@@ -410,8 +405,7 @@ brlconf_port_no_set (gint port_no)
 {
     if (!gnopiconf_set_int (port_no, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_PORT_NO))
     {
-        sru_warning (_("brlconf:Failed to set Serial Port no:%d\n"), port_no);
-        sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set serial port number: %d."), port_no);
     }
 }
 
@@ -421,8 +415,7 @@ brlconf_translation_table_set (const gchar *table)
 {
     if (!gnopiconf_set_string (table, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_TRANSLATION))
     {
-        sru_warning (_("brlconf:Failed to set Translation Table:%s\n"), table);
-        sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set translation table: %s."), table);
     }
 }
 
@@ -431,8 +424,7 @@ brlconf_optical_sensor_set (gint sensor)
 {
     if (!gnopiconf_set_int (sensor, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_OPTICAL_SENSOR))
     {
-        sru_warning (_("brlconf:Failed to set Optical Sensor:%d\n"), sensor);
-        sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set optical sensor: %d."), sensor);
     }
 }
 
@@ -441,8 +433,7 @@ brlconf_position_sensor_set (gint sensor)
 {
     if (!gnopiconf_set_int (sensor, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_POSITION_SENSOR))
     {
-        sru_warning (_("brlconf:Failed to set Position Sensor:%d\n"), sensor);
-        sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set position sensor: %d."), sensor);
     }
 }
 
@@ -451,8 +442,7 @@ brlconf_status_cell_set (const gchar *status)
 {
     if (!gnopiconf_set_string (status, CONFIG_PATH BRAILLE_KEY_PATH BRAILLE_STATUS_CELL))
     {
-        sru_warning (_("brlconf:Failed to set Status Cell:%s\n"), status);
-        sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+        sru_warning (_("Failed to set status cell: %s."), status);
     }
 }
 
