@@ -76,10 +76,10 @@ const struct
     gchar *file_name;
 } trans_table_list [] = 
 {
-    {N_("GERMAN"), 	"GE"},
-    {N_("US"),		"US"},
-    {N_("SPAIN"),	"SPAIN"},
-    {N_("SWEED"),	"SWEED"},
+    {N_("American English"),	"en_US"},
+    {N_("German"),		"de"},
+    {N_("Spanish"),		"es"},
+    {N_("Swedish"),		"sv"},
     {NULL,		NULL}
 };
 
@@ -153,7 +153,7 @@ static GtkWidget 	*rb_status_cell [ NO_ITEM_IN_STATUS_CELL ];
 		
 
 /**
- * Positiohn Switches 						
+ * Braille Mapping					
 **/
 static GtkWidget	*w_braille_key_mapping;
 static GtkWidget 	*w_braille_mapping_add_modify;
@@ -162,6 +162,8 @@ static GtkWidget 	*rb_optical_sensor [ NO_ITEM_IN_OPTICAL_SENSORS ];
 static GtkWidget	*tv_braille_keys;
 static GtkWidget	*tv_cmd_list;
 static GtkWidget	*cb_braille_key;
+static GtkWidget	*bt_braille_key_remove;
+static GtkWidget	*bt_braille_key_modify;
 
 extern GList 	*brldev_id_list;
 extern GList	*brldev_description_list;	
@@ -333,17 +335,19 @@ brlui_braille_device_combo_changed (GtkWidget *entry,
     {
 	min_val   = (gdouble)MIN_TTY_CONSOL_NO;
 	max_val   = (gdouble)MAX_TTY_CONSOL_NO;
-	port_text = _("X consol number:");
+	/*To translators: X = X Window System server */
+	port_text = _("X display's _terminal number:");
     }
     else
     {
     	min_val   = (gdouble)MIN_SERIAL_PORT_NO;
 	max_val   = (gdouble)MAX_SERIAL_PORT_NO;
-    	port_text = _("\tPort: ");
+    	port_text = _("_Port:");
     }
 
     gtk_spin_button_set_range (GTK_SPIN_BUTTON (sp_port), min_val, max_val);
     gtk_label_set_text (GTK_LABEL (lb_port), port_text);
+    gtk_label_set_use_underline (GTK_LABEL (lb_port), TRUE);
 }
 
 static void
@@ -394,6 +398,7 @@ brlui_set_handlers_braille_device (GladeXML *xml,
     glade_xml_signal_connect (xml,"on_combo-entry1_changed",		
 			    GTK_SIGNAL_FUNC (brlui_braille_device_combo_changed));
 
+    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_braille_device)->entry), FALSE);
 }
 
 
@@ -516,6 +521,22 @@ brlui_translation_table_changeing (Braille *braille_setting)
     g_free (key);
 }
 
+static gboolean
+brlui_search_equal_func (GtkTreeModel  *model,
+			 gint 		column,
+			 const gchar	*key,
+			 GtkTreeIter	*iter,
+			 gpointer	search_data)
+{
+    gchar *list_key = NULL;
+    
+    gtk_tree_model_get (model, 	iter,
+    			column, &list_key,
+                    	-1);
+
+    return g_strcasecmp (key, list_key) > 0; 
+}
+
 
 static GtkTreeModel*
 brlui_create_model (void)
@@ -591,6 +612,13 @@ brlui_set_handlers_translation_table (GladeXML *xml, Braille *braille_setting)
     selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW (tv_translation_table));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);    
     
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tv_translation_table), 
+				    TABLE_COLUMN);    
+    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tv_translation_table),
+					brlui_search_equal_func,
+					NULL, NULL);
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv_translation_table), TRUE);
+
 }
 
 gboolean 
@@ -1168,7 +1196,10 @@ brlui_add_modify_ok (void)
 	return FALSE;
 
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+	gn_show_message (_("No selected command!"));
 	return FALSE;
+    }
     
     if (current_item.current_is_new)
     {
@@ -1188,6 +1219,11 @@ brlui_add_modify_ok (void)
     	    gtk_list_store_append (GTK_LIST_STORE (model), 
 				    &current_item.current_iter);
 	}
+	else
+	{
+	    gn_show_message (_("Invalid device key!"));
+	    return FALSE;
+	}
     }
         
     (current_item.current_keyitem)->command_list =
@@ -1429,6 +1465,13 @@ brlui_set_handlers_braille_mapping_add_modify (GladeXML *xml)
     
     selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_cmd_list));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE); 
+    
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tv_cmd_list), 
+				    CMD_COLUMN);    
+    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tv_cmd_list),
+					brlui_search_equal_func,
+					NULL, NULL);
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv_cmd_list), TRUE);
 
 }
 
@@ -1493,7 +1536,7 @@ brlui_braille_key_mapping_changeing (Braille *braille_setting)
 
 
     removed_items_list =
-	cmdconf_remove_item_from_gconf_list (removed_items_list);
+	cmdconf_remove_items_from_gconf_list (removed_items_list);
 	
     removed_items_list = 
 	brlui_removed_item_list_free (removed_items_list);
@@ -1567,7 +1610,10 @@ brlui_braille_keys_modify (void)
     GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_braille_keys));
 
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+        gn_show_message (_("No selected item to modify!"));
 	return;
+    }
     
     sru_return_if_fail (GTK_IS_LIST_STORE (model));
     
@@ -1577,7 +1623,7 @@ brlui_braille_keys_modify (void)
 	
     if (!key || strlen (key) == 0)
     {
-	sru_message (_("brlui:Invalid selected item"));	    
+	sru_message (_("Invalid selected item!"));	    
 	g_free (key);
 	return;
     }
@@ -1644,8 +1690,11 @@ brlui_remove_keyitem_from_list (GSList 	*list,
     GtkTreeSelection *selection = gtk_tree_view_get_selection (tree_view);	    
     
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+        gn_show_message (_("No selected item to remove!"));
 	return list;
-	    
+    }	    
+    
     sru_return_val_if_fail (GTK_IS_LIST_STORE (model), list);
     
     gtk_tree_model_get (model, &iter,
@@ -1654,7 +1703,7 @@ brlui_remove_keyitem_from_list (GSList 	*list,
 	
     if (!key || strlen (key) == 0)
     {
-        sru_message (_("brlui:Invalid selected item"));
+        sru_message (_("Invalid selected item!"));
         g_free (key);
         return list;
     }
@@ -1694,6 +1743,18 @@ brlui_braille_keys_remove_clicked (GtkWidget *button,
 
 }
 
+static void
+brlui_braille_keys_selection_changed (GtkTreeSelection *selection,
+				      gpointer  	user_data)
+{
+    gboolean 	sensitive;
+    
+    sensitive = gtk_tree_selection_get_selected (selection, NULL, NULL);
+    
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_braille_key_remove), sensitive);
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_braille_key_modify), sensitive);
+}
+
 
 static GtkListStore*
 brlui_populate_list_store_with_keys (GtkListStore *store, 
@@ -1756,6 +1817,9 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
     gint i;
     gchar *name;
     w_braille_key_mapping = glade_xml_get_widget (xml, "w_braille_mapping");
+    bt_braille_key_modify = glade_xml_get_widget (xml, "bt_braille_key_modify");
+    bt_braille_key_remove = glade_xml_get_widget (xml, "bt_braille_key_remove");
+    tv_braille_keys	  = glade_xml_get_widget (xml, "tv_braille_keys");
     
     for (i = 0 ; i < NO_ITEM_IN_POSITION_SENSORS ; i++)
     {
@@ -1773,9 +1837,6 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
 	name = NULL;
     }
 
-    tv_braille_keys	= glade_xml_get_widget (xml, "tv_braille_keys");
-
-
     glade_xml_signal_connect (xml,"on_bt_braille_key_remove_clicked",  
 		GTK_SIGNAL_FUNC (brlui_braille_keys_remove_clicked));
 
@@ -1794,6 +1855,11 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
     
     gtk_tree_view_set_model (GTK_TREE_VIEW (tv_braille_keys), model);
     
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_braille_keys));
+    
+    g_signal_connect (selection, "changed",
+		      G_CALLBACK (brlui_braille_keys_selection_changed), NULL);
+    
     gtk_tree_sortable_set_sort_column_id ( GTK_TREE_SORTABLE (model), 
 					    KEYS_COLUMN, 
 					    GTK_SORT_ASCENDING);
@@ -1819,8 +1885,16 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
 							NULL);    
     gtk_tree_view_append_column (GTK_TREE_VIEW (tv_braille_keys), column);
         
-    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_braille_keys));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+    
+    brlui_braille_keys_selection_changed (selection, NULL);
+    
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tv_braille_keys), 
+				    KEYS_COLUMN);    
+    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tv_braille_keys),
+					brlui_search_equal_func,
+					NULL, NULL);
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv_braille_keys), TRUE);
 }
 
 
