@@ -42,10 +42,10 @@ PresentationStruct default_presentations[] =
 		{"braille",	"list",		"generic",	N_("%role%%name%%childcount%%1childcount?\"item\":\"items\"1%")},
 		{"braille",	"list-item",	"generic",	N_("%role%%name%%(shortcut)%")},		
 		{"braille",	"menu",		"generic",	N_("%role%%name%%childcount%%1childcount?\"item\":\"items\"1%")},
-		{"braille",	"menu-bar",	"generic",	N_("%role%%name%%childcount%%1childcount?\"item\":\"items\"1%")},
+		{"braille",	"menu-bar",	"generic",	N_("%role%%name%%children[%name%][%!1237!%]%")},
 		{"braille",	"menu-item",	"generic",	N_("%role%%name%%(accelerator)%%(shortcut)%%?state:enabled?\"\":\"(U)\"?%")},
 		{"braille",	"page-tab",	"generic",	N_("%role%%name%%(shortcut)%")},
-		{"braille",	"push-button",	"generic",	N_("%role%%!123478!%%name%%!145678!%%(shortcut)%")},
+		{"braille",	"push-button",	"generic",	N_("%role%%!123478!%%name%%!145678!%%(shortcut)%%?state:enabled?\"\":\"(U)\"?%")},
 		{"braille",	"radio-button",	"focus:",	N_("%role%%name%")},
 		{"braille",	"radio-menu-item","generic",	N_("%role%%name%%(accelerator)%%(shortcut)%%?state:checked?\"(C)\":\"( )\"?%%?state:enabled?\"\":\"(U)\"?%")},
 		{"braille",	"slider",	"generic",	N_("%role%%name%%value:crt%")},
@@ -82,7 +82,7 @@ PresentationStruct default_presentations[] =
 		{"speech",	"menu-bar",	"generic",	N_("%role<voice=role>%%childcount<voice=childcount>%%1childcount?\"item\":\"items\"<voice=system>1%%children[%name<voice=name>%][]%")},
 		{"speech", 	"menu-item", 	"generic", 	N_("%name<voice=name>%%role<voice=role>%%?shortcut?\"shortcut\":\"\"<voice=system>?%%shortcut<voice=shortcut>%%?accelerator?\"accelerator\":\"\"<voice=system>?%%accelerator<voice=shortcut>%%?state:enabled?\"available\":\"not available\"<voice=state>?%")},
 		{"speech", 	"page-tab", 	"generic", 	N_("%name<voice=name>%%role<voice=role>%%?shortcut?\"shortcut\":\"\"<voice=system>?%%shortcut<voice=shortcut>%")},
-		{"speech",	"push-button",	"generic",	N_("%name<voice=name>%%role<voice=role>%%?shortcut?\"shortcut\":\"\"<voice=system>?%%shortcut<voice=shortcut>%")},
+		{"speech",	"push-button",	"generic",	N_("%name<voice=name>%%role<voice=role>%%?shortcut?\"shortcut\":\"\"<voice=system>?%%shortcut<voice=shortcut>%%?state:enabled?\"available\":\"not available\"<voice=state>?%")},
 		{"speech", 	"radio-button",	"generic", 	N_("%name<voice=name>%%role<voice=role>%%?shortcut?\"shortcut\":\"\"<voice=system>?%%shortcut<voice=shortcut>%%?state:checked?\"checked\":\"unchecked\"<voice=state>?%")},
 		{"speech", 	"radio-menu-item","generic", 	N_("%name<voice=name>%%role<voice=role>%%?shortcut?\"shortcut\":\"\"<voice=system>?%%shortcut<voice=shortcut>%%?accelerator?\"accelerator\":\"\"<voice=system>?%%accelerator<voice=shortcut>%%?state:checked?\"checked\":\"unchecked\"<voice=state>?%%?state:enabled?\"available\":\"not available\"<voice=state>?%")},
 	    	{"speech",	"radio-menu-item","object:state-changed:checked",	N_("%?state:checked?\"checked\":\"unchecked\"<voice=state>?%")},
@@ -104,7 +104,7 @@ PresentationStruct default_presentations[] =
 		{"speech", 	"tree-item", 	"object:state-changed:expanded",N_("%?state:expanded?\"expanded\":\"\"<voice=state>?%%?state:collapsed?\"collapsed\":\"\"<voice=state>?%")},
 		
 		{"speech", 	"tool-tip", 	"tooltip:show",	N_("show tool tip<voice=system>%name<voice=name>%")},
-		{"speech", 	"tool-tip", 	"tooltip:hide",	N_("hide tool tip<voice=system>%name<voice=name>%")},
+		{"speech", 	"tool-tip", 	"tooltip:hide",	""},
 	
 		{"speech", 	"generic", 	"window:create",N_("create window<voice=system>%name<voice=name>%")},
 		{"speech", 	"generic", 	"window:destroy",N_("destroy window<voice=system>%name<voice=name>%")},
@@ -113,6 +113,7 @@ PresentationStruct default_presentations[] =
 		{"speech", 	"generic", 	"window:restore",N_("restore window<voice=system>%name<voice=name>%")},
 		{"speech", 	"generic", 	"window:maximize",N_("maximize window<voice=system>%name<voice=name>%")},
 		{"speech", 	"generic", 	"window:switch",N_("switch to window<voice=system>%name<voice=name>%")},
+		{"speech", 	"generic", 	"window:rename",("rename window<voice=system>%name<voice=name>%")},
 
 		{"magnifier", 	"text", 	"generic",	"%text:char<AlignmentX=\"0\" AlignmentY=\"0\">%"},
 		{"magnifier", 	"generic", 	"generic",	"%object<AlignmentX=\"0\" AlignmentY=\"0\">%"}
@@ -160,7 +161,7 @@ presconf_presentation_struct_new (const gchar *device,
     sru_return_val_if_fail (event, NULL);
     sru_return_val_if_fail (chunk, NULL);
     
-    new_val =  (PresentationStruct *) g_new0( PresentationStruct, 1 );
+    new_val =  (PresentationStruct *) g_new0 ( PresentationStruct, 1 );
     
     if (new_val == NULL)
     {
@@ -184,12 +185,15 @@ gboolean
 presconf_presentation_struct_free (PresentationStruct* value)
 {
     sru_return_val_if_fail (value, FALSE);
+    
     g_free (value->device);
     g_free (value->role);
     g_free (value->event);
     g_free (value->chunk);
     g_free (value);
+    
     value = NULL;
+    
     return TRUE;
 }
 
@@ -208,6 +212,7 @@ presconf_add_pres_item_in_list (const gchar *key, const gchar *chunk)
     
     sru_return_if_fail (key);
     sru_return_if_fail (chunk);
+    sru_return_if_fail (strlen (chunk));
 
     tokens = g_strsplit (key, "_", -1);
         
@@ -257,6 +262,7 @@ presconf_preload_list (void)
 	
 	list_item  = (GConfEntry*)elem->data;
 	list_value = gconf_entry_get_value (list_item);
+	
 	if (list_value->type == GCONF_VALUE_STRING)
 	    presconf_add_pres_item_in_list (gconf_entry_get_key (list_item), 
 					    gconf_value_get_string (list_value)
@@ -288,16 +294,49 @@ presconf_changes_end (void)
 void
 presconf_set_defaults (void)
 {
-    gint iter;
+    gint iter;    
+    GSList  *list  = NULL,
+	    *elem  = NULL;
+        
+    list = presconf_get_all_entries ();
+    elem = list;
+    
+    while (elem)
+    {
+	if (!gnopiconf_unset_key (gconf_entry_get_key ((GConfEntry*)elem->data)))
+	    sru_warning ("Can not unset key:%s", gconf_entry_get_key ((GConfEntry*)elem->data));
+	
+	elem = elem->next;
+    }
+    
+    if (list) 
+	g_slist_free (list);
+    
     for (iter = 0 ; iter < G_N_ELEMENTS (default_presentations) ; iter ++)
     {
+        GConfValue *value = NULL;
 	gchar *key = NULL;
+
 	key = presconf_create_key ( default_presentations[iter].device,
 			    	    default_presentations[iter].role,
 			    	    default_presentations[iter].event);
-	gnopiconf_set_string (default_presentations[iter].chunk, key);
+
+    
+	value = gconf_client_get_default_from_schema (gnopernicus_client, 
+						      key, NULL);
+
+        if (value)
+	{
+	    gnopiconf_set_string (gconf_value_get_string (value), key);
+	    gconf_value_free (value);
+	    value = NULL;
+	}
+	else
+	    gnopiconf_set_string (default_presentations[iter].chunk, key);
+
 	g_free (key);
     }
+    
     presconf_changes_end ();
 }
 
@@ -305,19 +344,26 @@ void
 presconf_save_list (void)
 {
     GSList *elem = NULL;
+    
+    sru_return_if_fail (pres_list);
+    
     elem = pres_list;
     
     while (elem)
     {
 	gchar *key = NULL;
 	PresentationStruct *value = (PresentationStruct *)elem->data;
+	
 	key = presconf_create_key ( value->device,
 			    	    value->role,
 			    	    value->event);
 	gnopiconf_set_string (value->chunk, key);
+	
 	g_free (key);
+	
 	elem = elem->next;
     }
+    
     presconf_changes_end ();
 }
 
@@ -336,30 +382,33 @@ presconf_append_item_to_list (const gchar *device,
 gboolean
 presconf_remove_from_list(const gchar *device,
 			  const gchar *role,
-			  const gchar *event,
-			  const gchar *chunk)
+			  const gchar *event)
 {
     GSList *elem;
+    
     sru_return_val_if_fail (device,FALSE);
     sru_return_val_if_fail (role,  FALSE);
     sru_return_val_if_fail (event, FALSE);
-    sru_return_val_if_fail (chunk, FALSE);
+    
     elem = pres_list;
+    
     while (elem)
     {
 	PresentationStruct *value = (PresentationStruct *)elem->data;
-	if (!strcmp (value->device,device) &&
-	    !strcmp (value->role, role)    &&
-	    !strcmp (value->event,event))
+	
+	if (!strcmp (value->device, device) &&
+	    !strcmp (value->role,   role)    &&
+	    !strcmp (value->event,  event))
 	{
-	    pres_list = g_slist_remove_link (pres_list, elem);
-	    pres_removed_list = g_slist_prepend (pres_removed_list, elem->data);
+	    pres_list 		= g_slist_remove_link (pres_list, elem);
+	    pres_removed_list 	= g_slist_prepend (pres_removed_list, elem->data);
 	    g_slist_free (elem);
 	    return TRUE;
 	} 
 		 
 	elem = elem->next;
     }
+    
     return FALSE;
 }
 
@@ -368,7 +417,9 @@ void
 presconf_unset_remove_list (void)
 {
     GSList *elem;
+    
     elem = pres_removed_list;
+    
     while (elem)
     {
 	gchar *key;
@@ -376,10 +427,12 @@ presconf_unset_remove_list (void)
 	key = presconf_create_key ( value->device,
 			    	    value->role,
 			    	    value->event);
-	gnopiconf_unset_key (key);
+	/*gnopiconf_unset_key (key);*/
+	gnopiconf_set_string (NULL, key);
 	g_free (key);
 	elem = elem->next;
     }
+    
     presconf_free_list (pres_removed_list);
     pres_removed_list = NULL;
 }
@@ -390,13 +443,17 @@ presconf_item_exist(const gchar *device,
 		    const gchar *event)
 {
     GSList *elem;
+    
     sru_return_val_if_fail (device, FALSE);
     sru_return_val_if_fail (role,   FALSE);
     sru_return_val_if_fail (event,  FALSE);
+    
     elem = pres_list;
+    
     while (elem)
     {
 	PresentationStruct *value = (PresentationStruct *)elem->data;
+	
 	if (!strcmp (value->device,device) &&
 	    !strcmp (value->role, role)    &&
 	    !strcmp (value->event,event))
@@ -406,6 +463,7 @@ presconf_item_exist(const gchar *device,
 		 
 	elem = elem->next;
     }
+    
     return FALSE;
 }
 
@@ -415,13 +473,17 @@ presconf_search_item (const gchar *device,
 		      const gchar *event)
 {
     GSList *elem;
+    
     sru_return_val_if_fail (device, NULL);
     sru_return_val_if_fail (role, NULL);
     sru_return_val_if_fail (event, NULL);
+    
     elem = pres_list;
+    
     while (elem)
     {
 	PresentationStruct *value = (PresentationStruct *)elem->data;
+	
 	if (!strcmp (value->device,device) &&
 	    !strcmp (value->role, role)    &&
 	    !strcmp (value->event,event))
@@ -431,6 +493,7 @@ presconf_search_item (const gchar *device,
 		 
 	elem = elem->next;
     }
+    
     return NULL;
 }
 
