@@ -294,6 +294,7 @@ presui_get_chunk (void)
     {
 	gchar *key = NULL;
 	gchar *tmp = NULL;
+	
 	if (GTK_IS_LIST_STORE (model))
 	{
 	    gtk_tree_model_get (model, 	&iter,
@@ -309,6 +310,7 @@ presui_get_chunk (void)
 	    g_free (str);
 	    str = tmp;
 	}
+	
 	g_free (key);
 	valid = gtk_tree_model_iter_next (model, &iter);
     }
@@ -320,6 +322,7 @@ static void
 presui_refresh_entry (void)
 {
     gchar *chunk = presui_get_chunk ();
+    
     if (chunk && strlen (chunk) > 0)
 	gtk_entry_set_text (GTK_ENTRY (et_pres_add_chunks), chunk); 
     else
@@ -356,8 +359,17 @@ presui_pres_update_list (void)
 	chunk = presui_get_chunk ();
 	
 	if (!chunk)
-	    chunk = "";
+	{
+	    if (presconf_remove_from_list (device, role, event))
+		gtk_list_store_remove (GTK_LIST_STORE (model), &current_iter);
+	    
+	    return;
+	}
 	
+	/*
+	if (!chunk)
+	    chunk = g_strdup ("");
+	*/
 	if (new_item_in_list)
 	{
 	    if (!presconf_item_exist (device, role, event))
@@ -465,6 +477,7 @@ presui_pres_chunk_add_new_clicked (GtkWidget	*widget,
 {
     GtkTreeIter 	iter;
     GtkTreeModel 	*model; 	
+    gchar 		*et_chunk;
     model = gtk_tree_view_get_model ( GTK_TREE_VIEW (tv_pres_add_chunk) );
     
     presui_presentation_add_apply_sensitivity (TRUE);
@@ -476,7 +489,13 @@ presui_pres_chunk_add_new_clicked (GtkWidget	*widget,
         		PRESENTATION_ADD_LIST,  "%%",
 			PRESENTATION_EDITABLE,  TRUE,
 			-1);
-    presui_refresh_entry ();
+			
+    et_chunk = (gchar*)	gtk_entry_get_text (GTK_ENTRY (et_pres_add_chunks)); 
+    et_chunk = g_strdup_printf ("%s%s", et_chunk, "%%");
+    gtk_entry_set_text (GTK_ENTRY (et_pres_add_chunks), et_chunk); 
+    
+    g_free (et_chunk);
+//  presui_refresh_entry ();
 }
 
 
@@ -496,6 +515,7 @@ presui_pres_chunk_remove_clicked (GtkWidget	*widget,
     
     if (gtk_tree_selection_get_selected (selection, NULL, &iter))
 	gtk_list_store_remove ( GTK_LIST_STORE (model), &iter);
+	
     presui_refresh_entry ();
 }
 
@@ -513,6 +533,7 @@ presui_cell_edited (GtkCellRendererText *cell,
     gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
         		PRESENTATION_ADD_LIST,  new_text,
 			-1);
+			
     presui_refresh_entry ();
 }
 
@@ -612,6 +633,7 @@ presui_pres_chunk_move_clicked (GtkWidget *widget,
 	    }
 	    
 	    gtk_tree_selection_select_iter (selection, &next_iter);
+	    
 	    presui_refresh_entry ();
 	}
     }
@@ -1032,9 +1054,9 @@ presui_pres_remove_clicked (GtkWidget 	*widget,
     GtkTreeModel 	*model; 	
     GtkTreeSelection 	*selection;
     model 	= 
-	gtk_tree_view_get_model ( GTK_TREE_VIEW (tv_presentation_order) );
+	gtk_tree_view_get_model (GTK_TREE_VIEW (tv_presentation_order) );
     selection 	= 
-	gtk_tree_view_get_selection ( GTK_TREE_VIEW (tv_presentation_order) );	    
+	gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_presentation_order) );	    
     
     presui_apply_sensitivity (TRUE);
     
@@ -1054,9 +1076,8 @@ presui_pres_remove_clicked (GtkWidget 	*widget,
 				PRESENTATION_CHUNK,  &chunk,
             			-1);
 
-	    gtk_list_store_remove ( GTK_LIST_STORE (model), &iter);
-	    presconf_remove_from_list (device, role,
-				      event, chunk);
+	    gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+	    presconf_remove_from_list (device, role, event);
 	}
     }
     else
@@ -1078,6 +1099,7 @@ presui_pres_modify_clicked (GtkWidget 	*widget,
 
     new_item_in_list = FALSE;
     presui_apply_sensitivity (TRUE);
+    
     if (!presui_load_presentation_add ())
 	sru_warning (_("We could not load interface."));
 }
@@ -1236,7 +1258,7 @@ presui_set_handlers (GladeXML *xml)
 							NULL);	
     gtk_tree_view_column_set_clickable (column, TRUE);
     gtk_tree_view_column_set_resizable (column, TRUE);
-    gtk_tree_view_column_set_max_width (column, 70);
+    gtk_tree_view_column_set_max_width (column, 80);
     gtk_tree_view_append_column (GTK_TREE_VIEW (tv_presentation_order), column);
     
     column = gtk_tree_view_column_new_with_attributes   (_("Event"),
@@ -1245,7 +1267,7 @@ presui_set_handlers (GladeXML *xml)
 							NULL);	
     gtk_tree_view_column_set_clickable (column, TRUE);
     gtk_tree_view_column_set_resizable (column, TRUE);
-    gtk_tree_view_column_set_max_width (column, 70);
+    gtk_tree_view_column_set_max_width (column, 80);
     
     gtk_tree_view_append_column (GTK_TREE_VIEW (tv_presentation_order), column);
         
