@@ -51,6 +51,8 @@ enum
 struct {
     GtkWidget	*w_voice_setting;
     GtkWidget	*tv_voice_table;
+    GtkWidget	*bt_voice_remove;
+    GtkWidget	*bt_voice_modify;
 } spui_speech_voice;
 
 struct {
@@ -103,8 +105,6 @@ static GnopernicusSpeakerListType	*sp_curr_gnopernicus_speaker	= NULL;
 static SpeakerSettingsListType  	*sp_speakers_settings		= NULL;
 static SpeakerSettingsListType  	*sp_speakers_settings_backup	= NULL;
 static SpeakerSettingsListType		*sp_curr_speaker_settings	= NULL;
-static SpeakerSettingsListType		*sp_speakers_settings_abs_backup= NULL;
-static SpeakerSettingsListType		*sp_speakers_settings_rel_backup= NULL;
 static SpeechDriverListType		*sp_speech_driver_list		= NULL;
 
 #if 0
@@ -121,8 +121,8 @@ spui_debug (GSList *list)
 	{
 	    if (ss->gnopernicus_speaker)
 		fprintf (stderr,"\ngn_speker:%s", ss->gnopernicus_speaker);
-	    if (ss->driver_voice)
-		fprintf (stderr,"\ndriver_voice:%s", ss->driver_voice);
+	    if (ss->voice_name)
+		fprintf (stderr,"\nvoice_name:%s", ss->voice_name);
 	    if (ss->driver_name)
 		fprintf (stderr,"\ndriver_name:%s", ss->driver_name);
 	    fprintf (stderr,"\nvolume:%d", ss->volume);
@@ -135,29 +135,23 @@ spui_debug (GSList *list)
 }
 #endif
 
-void 
-spui_send_msg (const gchar *msg)
-{
-    GtkWidget *dialog = NULL;
-    dialog = gtk_message_dialog_new (
-	    NULL,
-	    GTK_DIALOG_DESTROY_WITH_PARENT,
-	    GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
-	    "%s",
-	    msg);
-    gtk_dialog_run (GTK_DIALOG (dialog));
-    gtk_widget_destroy (dialog);
-}
-
 static void
 spui_voice_old_speaker_setting_clean (void)
 {
     sp_old_speaker_settings.volume = 0;
     sp_old_speaker_settings.rate   = 0;
     sp_old_speaker_settings.pitch  = 0;
-    g_free (sp_old_speaker_settings.driver_voice);
+    
+    g_free (sp_old_speaker_settings.driver_name);
+    g_free (sp_old_speaker_settings.driver_name_descr);
+    g_free (sp_old_speaker_settings.voice_name);
+    g_free (sp_old_speaker_settings.voice_name_descr);
     g_free (sp_old_speaker_settings.gnopernicus_speaker);
-    sp_old_speaker_settings.driver_voice = NULL;
+    
+    sp_old_speaker_settings.driver_name 	= NULL;
+    sp_old_speaker_settings.driver_name_descr 	= NULL;
+    sp_old_speaker_settings.voice_name 		= NULL;
+    sp_old_speaker_settings.voice_name_descr 	= NULL;
     sp_old_speaker_settings.gnopernicus_speaker = NULL;
 }
 
@@ -171,8 +165,8 @@ spui_voice_speaker_parameter_in_list_store_at_iter_set (SpeakerSettings *item,
 
     gtk_list_store_set (GTK_LIST_STORE (store), &iter, 
 			VOICE_GNOPERNICUS_SPEAKER,	(gchar*)item->gnopernicus_speaker,
-			VOICE_DRIVER_NAME,		(gchar*)item->driver_name,
-			VOICE_DRIVER_VOICE,  		(gchar*)item->driver_voice,
+			VOICE_DRIVER_NAME,		(gchar*)item->driver_name_descr,
+			VOICE_DRIVER_VOICE,  		(gchar*)item->voice_name_descr,
 			VOICE_VOLUME,			item->volume,
 			VOICE_RATE,			item->rate,
 			VOICE_PITCH,			item->pitch,
@@ -214,6 +208,9 @@ spui_voice_iter_for_speaker_get (const gchar *key,
     return FALSE;
 }
 
+
+
+
 /************************************************************************/
 static SpeakerSettings*
 spui_voice_add_modify_new_speaker_settings_get (void)
@@ -227,8 +224,11 @@ spui_voice_add_modify_new_speaker_settings_get (void)
     new_settings->volume 		= CHECK_RANGE_OF_VALUE (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spui_speech_voice_add_modify.sp_voice_volume)), MIN_SPEECH_VOLUME, MAX_SPEECH_VOLUME);
     new_settings->rate   		= CHECK_RANGE_OF_VALUE (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spui_speech_voice_add_modify.sp_voice_rate)), MIN_SPEECH_RATE, MAX_SPEECH_RATE);
     new_settings->pitch  		= CHECK_RANGE_OF_VALUE (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spui_speech_voice_add_modify.sp_voice_pitch)), MIN_SPEECH_PITCH, MAX_SPEECH_PITCH);
-    new_settings->driver_voice 		= g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_voice)->entry)));
-    new_settings->driver_name 		= g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_driver)->entry)));
+    new_settings->voice_name 		= g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_voice)->entry)));
+    new_settings->voice_name_descr	= spconf_driver_list_voice_descr_name_get (new_settings->voice_name);
+    new_settings->driver_name_descr	= g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_driver)->entry)));
+    new_settings->driver_name 		= g_strdup (spconf_driver_list_get_driver_name (sp_speech_driver_list,
+											new_settings->driver_name_descr));
     new_settings->gnopernicus_speaker 	= g_strdup (gtk_entry_get_text (GTK_ENTRY (spui_speech_voice_add_modify.et_gnopernicus_voice)));    
     
     return new_settings;
@@ -274,10 +274,11 @@ spui_voice_add_modify_properties_get (void)
 						    sp_curr_gnopernicus_speaker->data);
 	gtk_list_store_remove (GTK_LIST_STORE (model), &sp_curr_iter);
 
-	spui_send_msg (_("Invalide voice!"));
+	gn_show_message (_("Invalid voice!"));
 	    
 	return FALSE;
     }
+    
     spui_voice_speaker_parameter_in_list_store_at_iter_set (new_setting, 
 						    	    GTK_LIST_STORE (model),
 					    	    	    sp_curr_iter);
@@ -323,6 +324,7 @@ spui_voice_add_modify_combo_list_set (GList *list, GtkWidget *combo)
 {
     if (!list) 
 	return;
+	
     gtk_combo_set_popdown_strings (GTK_COMBO (combo), list);
 }
 
@@ -342,7 +344,7 @@ spui_voice_add_modify_speaker_values_modify (SpeakerSettings *settings)
     spui_voice_add_modify_combo_list_set (tmp, spui_speech_voice_add_modify.cb_engine_driver);
     spconf_list_for_combo_free (tmp);
     
-    tmp = spconf_voice_list_for_combo_get (sp_speech_driver_list, settings->driver_name, NULL);
+    tmp = spconf_voice_list_for_combo_get (sp_speech_driver_list, settings->driver_name_descr, NULL);
     spui_voice_add_modify_combo_list_set (tmp, spui_speech_voice_add_modify.cb_engine_voice);
     spconf_list_for_combo_free (tmp);
 
@@ -354,8 +356,8 @@ spui_voice_add_modify_speaker_values_modify (SpeakerSettings *settings)
     gtk_range_set_value (GTK_RANGE (spui_speech_voice_add_modify.hs_voice_rate), 	settings->rate);
     gtk_range_set_value (GTK_RANGE (spui_speech_voice_add_modify.hs_voice_pitch), 	settings->pitch);
     gtk_entry_set_text  (GTK_ENTRY (spui_speech_voice_add_modify.et_gnopernicus_voice), settings->gnopernicus_speaker);
-    gtk_entry_set_text  (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_voice)->entry), settings->driver_voice);
-    gtk_entry_set_text  (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_driver)->entry), settings->driver_name);
+    gtk_entry_set_text  (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_voice)->entry), settings->voice_name_descr);
+    gtk_entry_set_text  (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_driver)->entry), settings->driver_name_descr);
     
     return TRUE;
 }
@@ -371,8 +373,10 @@ spui_voice_add_modify_properties_settings_set (SpeakerSettings *settings)
     sp_old_speaker_settings.volume 		= settings->volume;
     sp_old_speaker_settings.rate   		= settings->rate;
     sp_old_speaker_settings.pitch  		= settings->pitch;
-    sp_old_speaker_settings.driver_voice 	= g_strdup (settings->driver_voice);
+    sp_old_speaker_settings.voice_name 		= g_strdup (settings->voice_name);
+    sp_old_speaker_settings.voice_name_descr 	= g_strdup (settings->voice_name_descr);
     sp_old_speaker_settings.driver_name 	= g_strdup (settings->driver_name);
+    sp_old_speaker_settings.driver_name_descr 	= g_strdup (settings->driver_name_descr);
     sp_old_speaker_settings.gnopernicus_speaker = g_strdup (settings->gnopernicus_speaker);
     
     return spui_voice_add_modify_speaker_values_modify (settings);
@@ -411,6 +415,7 @@ spui_voice_add_modify_hs_value_changed (GtkWidget	*widget,
     	
 }
 
+
 static void
 spui_voice_add_modify_sp_value_changed (GtkWidget	*widget,
 				    	gpointer	data)
@@ -444,14 +449,26 @@ spui_voice_add_modify_sp_value_changed (GtkWidget	*widget,
 	gtk_range_set_value (GTK_RANGE ((GtkWidget*)data), (gdouble) value1);	
 }
 
+static gboolean
+spui_voice_add_modify_sp_value_output (GtkWidget	*widget,
+				    	    gpointer	data)
+
+{
+    spui_voice_add_modify_sp_value_changed (widget, data);
+
+    return FALSE;
+}
+
+
 static void
 spui_voice_add_modify_driver_combo_changed (GtkWidget *widget,
 		        		    gpointer  user_data)
 {
     const gchar  	*new_driver_name;
     const SpeechDriver  *driver;
+    const SpeechVoice	*sp_voice;
     SpeakerSettings 	*curr_setting = NULL;
-    GList		*list = NULL;
+    GList		*sp_list = NULL;
     
     if (!sp_curr_speaker_settings ||
 	!sp_changes_activable) 
@@ -466,28 +483,52 @@ spui_voice_add_modify_driver_combo_changed (GtkWidget *widget,
     if (!new_driver_name ||
 	!strlen (new_driver_name))
 	return;
-	
-    driver = spconf_driver_list_get_driver (sp_speech_driver_list,
-					    new_driver_name);
 
+    /* get a new driver */	
+    driver = spconf_driver_list_get_driver_from_name_descr (sp_speech_driver_list,
+						            new_driver_name);
+
+    /* verify if it is a valid driver */
     if (!driver)
 	return;
 	
     if (!strcmp (driver->driver_name, 
-		curr_setting->driver_name))
+		 curr_setting->driver_name))
 	return;
-	
-    list = spconf_voice_list_for_combo_get (sp_speech_driver_list, 
-					    driver->driver_name, 
+
+    /* get list of speaker for driver name */	
+    sp_list = spconf_voice_list_for_combo_get (sp_speech_driver_list, 
+					    driver->driver_name_descr, 
 					    NULL);
 
-    spui_voice_add_modify_combo_list_set (list, spui_speech_voice_add_modify.cb_engine_voice); 
+    /* verify if the driver has a speaker */
+    if (!sp_list)
+	return;
 
-    spconf_list_for_combo_free (list);
+    /* set speaker list in combo box */
+    spui_voice_add_modify_combo_list_set (sp_list, spui_speech_voice_add_modify.cb_engine_voice); 
+    	
+    /* get speaker voice */
+    sp_voice  = spconf_driver_list_get_speech_voice (sp_speech_driver_list,
+						     driver->driver_name,
+						     (gchar*)sp_list->data);
 
+    /* free speaker list */
+    spconf_list_for_combo_free (sp_list);
+
+    /* free current driver and speaker values */
     g_free (curr_setting->driver_name);
-    curr_setting->driver_name = g_strdup (driver->driver_name);
+    g_free (curr_setting->driver_name_descr);
+    g_free (curr_setting->voice_name);
+    g_free (curr_setting->voice_name_descr);
     
+    /* set current driver and speaker values */
+    curr_setting->voice_name 	    = g_strdup (sp_voice->voice_name);
+    curr_setting->voice_name_descr  = g_strdup (sp_voice->voice_name_descr);
+    curr_setting->driver_name 	    = g_strdup (driver->driver_name);
+    curr_setting->driver_name_descr = g_strdup (driver->driver_name_descr);
+    
+    /* save new settings if it is a new item*/
     if (!sp_new_item)
 	 spconf_speaker_settings_save (curr_setting);
 }
@@ -497,33 +538,37 @@ spui_voice_add_modify_speaker_combo_changed (GtkWidget *widget,
 		        		     gpointer  user_data)
 {
 
-    gchar *cvalue;
-    SpeakerSettings *curr_setting = NULL;
+    const gchar 	*voice_name;
+    SpeakerSettings 	*curr_setting = NULL;
+    const SpeechVoice	*sp_voice;
     
     if (!sp_curr_speaker_settings ||
 	!sp_changes_activable) 
 	return;
     
-    curr_setting = 
-	(SpeakerSettings*)sp_curr_speaker_settings->data;
-    (const gchar*)cvalue = 
-	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_voice)->entry));
+    curr_setting = (SpeakerSettings*)sp_curr_speaker_settings->data;
+    voice_name   = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_add_modify.cb_engine_voice)->entry));
 
-    if (!strcmp (curr_setting->driver_voice, cvalue))
+    if (!strcmp (curr_setting->voice_name_descr, voice_name)) 
+	return;
+
+    sp_voice  = spconf_driver_list_get_speech_voice (sp_speech_driver_list,
+						     curr_setting->driver_name,
+						     voice_name);
+
+    if (!sp_voice)
 	return;
-    
-    if (strcmp (curr_setting->driver_voice, cvalue) && 
-	strlen (cvalue) > 0 &&
-	strcmp (cvalue, NONE_ELEMENT) &&
-	strcmp (curr_setting->gnopernicus_speaker, NONE_ELEMENT))
-    {
-	g_free (curr_setting->driver_voice);
-	curr_setting->driver_voice = g_strdup (cvalue);
 	
-	if (!sp_new_item)
-	    spconf_speaker_settings_save (curr_setting);
-    }
-    
+    if (!strcmp (curr_setting->gnopernicus_speaker, NONE_ELEMENT))
+	return;
+
+    g_free (curr_setting->voice_name);
+    g_free (curr_setting->voice_name_descr);
+    curr_setting->voice_name 		= g_strdup (sp_voice->voice_name);
+    curr_setting->voice_name_descr 	= g_strdup (sp_voice->voice_name_descr);
+
+    if (!sp_new_item)
+        spconf_speaker_settings_save (curr_setting);
 }
 
 static void
@@ -614,12 +659,23 @@ spui_voice_add_modify_handlers_set  (GladeXML *xml)
     glade_xml_signal_connect_data (xml, "on_sp_voice_volume_changed",			
 			    GTK_SIGNAL_FUNC (spui_voice_add_modify_sp_value_changed), 
 			    (gpointer)spui_speech_voice_add_modify.hs_voice_volume);
+    glade_xml_signal_connect_data (xml, "on_sp_voice_volume_output",			
+			    GTK_SIGNAL_FUNC (spui_voice_add_modify_sp_value_output), 
+			    (gpointer)spui_speech_voice_add_modify.hs_voice_volume);
+
     glade_xml_signal_connect_data (xml, "on_sp_voice_rate_changed",			
 			    GTK_SIGNAL_FUNC (spui_voice_add_modify_sp_value_changed), 
 			    (gpointer)spui_speech_voice_add_modify.hs_voice_rate);
+    glade_xml_signal_connect_data (xml, "on_sp_voice_rate_output",			
+			    GTK_SIGNAL_FUNC (spui_voice_add_modify_sp_value_output), 
+			    (gpointer)spui_speech_voice_add_modify.hs_voice_rate);
+
     glade_xml_signal_connect_data (xml, "on_sp_voice_pitch_changed",			
 			    GTK_SIGNAL_FUNC (spui_voice_add_modify_sp_value_changed),
 			    (gpointer)spui_speech_voice_add_modify.hs_voice_pitch);
+    glade_xml_signal_connect_data (xml, "on_sp_voice_pitch_output",			
+			    GTK_SIGNAL_FUNC (spui_voice_add_modify_sp_value_output), 
+			    (gpointer)spui_speech_voice_add_modify.hs_voice_pitch);
 
 }
 
@@ -645,6 +701,9 @@ spui_voice_add_modify_load (GtkWidget *parent_window)
     return TRUE;
 }
 
+
+
+
 /******************************************************************************/
 static void
 spui_voice_relative_modify_for_all_voices  (gint volume,
@@ -655,26 +714,27 @@ spui_voice_relative_modify_for_all_voices  (gint volume,
     gboolean all_drivers = FALSE;
     gboolean all_voices = FALSE;
     const gchar *driver_name;
-    const gchar *driver_voice;
+    const gchar *voice_name;
     
     driver_name = 
 	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_relative.cb_rel_engine_driver)->entry));    
-    driver_voice = 	
+    voice_name = 	
 	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_relative.cb_rel_engine_voice)->entry));    
     
     all_drivers = !strcmp (driver_name, SPEECH_ALL_DRIVERS);
-    all_voices  = !strcmp (driver_voice, SPEECH_ALL_VOICES);
+    all_voices  = !strcmp (voice_name, SPEECH_ALL_VOICES);
     
     for (tmp = sp_speakers_settings; tmp ; tmp = tmp->next)
     {
 	SpeakerSettings *sp_set = (SpeakerSettings *)tmp->data;
 	if (!all_drivers)
 	{
-	    if (strcmp (sp_set->driver_name, driver_name))
+	    if (strcmp (sp_set->driver_name_descr, driver_name))
 		continue;
+		
 	    if (!all_voices)
 	    {
-		if (strcmp (sp_set->driver_voice, driver_voice))
+		if (strcmp (sp_set->voice_name_descr, voice_name))
 		    continue;
 	    }
 	}
@@ -760,11 +820,9 @@ spui_voice_relative_response (GtkDialog *dialog,
 {
     switch (response_id)
     {
+	case GTK_RESPONSE_CLOSE:
 	case GTK_RESPONSE_OK: 
         break;
-	case GTK_RESPONSE_CANCEL:
-	    spconf_speaker_settings_list_save (sp_speakers_settings_rel_backup);
-        break;
 	case GTK_RESPONSE_HELP: 
 	    gn_load_help ("gnopernicus-speech-settings");
 	    return;
@@ -772,8 +830,6 @@ spui_voice_relative_response (GtkDialog *dialog,
 	default:
         break;
     }
-    sp_speakers_settings_rel_backup =
-	    spconf_speaker_settings_list_free (sp_speakers_settings_rel_backup);
     gtk_widget_hide ((GtkWidget*)dialog);
 }
 
@@ -796,8 +852,8 @@ spui_voice_relative_handlers_set  (GladeXML *xml)
 {
     spui_speech_voice_relative.w_voice_setting_relative_values = glade_xml_get_widget (xml, "w_voice_setting_with_relative_values");
     
-    spui_speech_voice_relative.cb_rel_engine_voice 	   = glade_xml_get_widget (xml, "cb_rel_voice");
-    spui_speech_voice_relative.cb_rel_engine_driver 	   = glade_xml_get_widget (xml, "cb_rel_driver");
+    spui_speech_voice_relative.cb_rel_engine_voice 	= glade_xml_get_widget (xml, "cb_rel_voice");
+    spui_speech_voice_relative.cb_rel_engine_driver 	= glade_xml_get_widget (xml, "cb_rel_driver");
     
     g_signal_connect (spui_speech_voice_relative.w_voice_setting_relative_values, "response",
 		      G_CALLBACK (spui_voice_relative_response), NULL);
@@ -833,11 +889,6 @@ spui_voice_relative_handlers_set  (GladeXML *xml)
 static gboolean 
 spui_voice_relative_load (GtkWidget *parent_window)
 {         
-    sru_assert (!sp_speakers_settings_rel_backup);
-
-    sp_speakers_settings_rel_backup =
-	    spconf_speaker_settings_list_clone (sp_speakers_settings);
-
     if (!spui_speech_voice_relative.w_voice_setting_relative_values)
     {
 	GladeXML *xml;
@@ -858,7 +909,22 @@ spui_voice_relative_load (GtkWidget *parent_window)
     
     return TRUE;
 }
+
+
+
+
 /******************************************************************************/
+static void
+spui_voice_absolute_properties_sensitivity_set (gboolean sensitivity)
+{
+    gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.ck_abs_volume),  
+			    sensitivity);
+    gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.ck_abs_rate),  
+			    sensitivity);
+    gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.ck_abs_pitch),  
+			    sensitivity);
+}
+
 static void
 spui_voice_absolute_save_modify (gint volume,
 				 gint rate,
@@ -867,29 +933,31 @@ spui_voice_absolute_save_modify (gint volume,
     SpeakerSettingsListType *tmp = NULL;
     
     const gchar *driver_name;
-    const gchar *driver_voice;
+    const gchar *voice_name;
     gboolean all_drivers = FALSE;
     gboolean all_voices = FALSE;
     
     
     driver_name = 
 	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_driver)->entry));    
-    driver_voice = 	
+    voice_name = 	
     	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_voice)->entry));    
     
     all_drivers = !strcmp (driver_name, SPEECH_ALL_DRIVERS);
-    all_voices  = !strcmp (driver_voice, SPEECH_ALL_VOICES);
+    all_voices  = !strcmp (voice_name, SPEECH_ALL_VOICES);
     
     for (tmp = sp_speakers_settings; tmp ; tmp = tmp->next)
     {
 	SpeakerSettings *sp_set = (SpeakerSettings *)tmp->data;
 	if (!all_drivers)
 	{
-	    if (strcmp (sp_set->driver_name, driver_name))
+
+	    if (strcmp (sp_set->driver_name_descr, driver_name))
 		continue;
+
 	    if (!all_voices)
 	    {
-		if (strcmp (sp_set->driver_voice, driver_voice))
+		if (strcmp (sp_set->voice_name_descr, voice_name))
 		    continue;
 	    }
 	}
@@ -907,7 +975,7 @@ spui_voice_absolute_save_modify (gint volume,
 
 static gboolean
 spui_voice_absolute_median_values_return (const gchar *driver_name,
-					  const gchar *driver_voice,
+					  const gchar *voice_name,
 					  gint	*volume,
 					  gint	*rate,
 					  gint	*pitch)
@@ -922,7 +990,7 @@ spui_voice_absolute_median_values_return (const gchar *driver_name,
     gint mpitch  = 0;
     
     all_drivers = !strcmp (driver_name, SPEECH_ALL_DRIVERS);
-    all_voices  = !strcmp (driver_voice, SPEECH_ALL_VOICES);
+    all_voices  = !strcmp (voice_name, SPEECH_ALL_VOICES);
     
     count = 0;
     
@@ -936,19 +1004,20 @@ spui_voice_absolute_median_values_return (const gchar *driver_name,
 
 	if (!all_drivers)
 	{
-	    if (strcmp (sp_set->driver_name, driver_name))
+	    if (strcmp (sp_set->driver_name_descr, driver_name))
 		continue;
+		
 	    if (!all_voices)
 	    {
-		if (strcmp (sp_set->driver_voice, driver_voice))
+		if (strcmp (sp_set->voice_name_descr, voice_name))
 		    continue;
 	    }
 	}
 	
-	count = count + 1;
+	count   = count   + 1;
 	mvolume = mvolume + sp_set->volume;
-	mrate 	= mrate + sp_set->rate;
-	mpitch	= mpitch + sp_set->pitch;
+	mrate 	= mrate   + sp_set->rate;
+	mpitch	= mpitch  + sp_set->pitch;
     }
 
     if (!count)
@@ -965,18 +1034,18 @@ static void
 spui_voice_absolute_set_values_to_parameters (void)
 {
     const gchar *driver_name;
-    const gchar *driver_voice;
+    const gchar *voice_name;
     gint volume = 0;
     gint rate   = 0;
     gint pitch  = 0;
     
     driver_name = 
 	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_driver)->entry));    
-    driver_voice = 	
+    voice_name = 	
     	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_voice)->entry));    
 
     if (spui_voice_absolute_median_values_return (driver_name,
-					          driver_voice,
+					          voice_name,
 					          &volume,
 					    	  &rate,
 					          &pitch))
@@ -989,6 +1058,11 @@ spui_voice_absolute_set_values_to_parameters (void)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spui_speech_voice_absolute.sp_abs_volume), (gdouble)volume);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spui_speech_voice_absolute.sp_abs_rate),   (gdouble)rate);
 	sp_abs_changes_activable = TRUE;
+	spui_voice_absolute_properties_sensitivity_set (TRUE);
+    }
+    else
+    {
+	spui_voice_absolute_properties_sensitivity_set (FALSE);
     }
 }
 
@@ -996,6 +1070,7 @@ static void
 spui_voice_absolute_clear_all_param (void)
 {
     GList *tmp = NULL;
+    GtkWidget *label;
     
     spui_speech_voice_absolute.abs_volume_sens = FALSE;
     spui_speech_voice_absolute.abs_rate_sens = FALSE;
@@ -1007,6 +1082,34 @@ spui_voice_absolute_clear_all_param (void)
 				    spui_speech_voice_absolute.abs_rate_sens);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (spui_speech_voice_absolute.ck_abs_pitch), 
 				    spui_speech_voice_absolute.abs_pitch_sens);
+
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_volume),
+				"abs_volume");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_volume_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_volume),
+				"abs_volume_perc");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_volume_sens);
+
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_rate),
+				"abs_rate");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_rate_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_rate),
+				"abs_rate_word_min");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_rate_sens);
+
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_pitch),
+				"abs_pitch");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_pitch_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_pitch),
+				"abs_pitch_hz");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_pitch_sens);
+			      
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.sp_abs_volume), 
 			      spui_speech_voice_absolute.abs_volume_sens);
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.hs_abs_volume), 
@@ -1034,38 +1137,54 @@ spui_voice_absolute_force_all_clicked (GtkWidget 	*widget,
 			    	       gpointer		user_data)
 {
     SpeakerSettingsListType *tmp = NULL;
-    const gchar *driver_name;
-    const gchar *driver_voice;
+    const gchar 	 *driver_name;
+    const gchar 	 *voice_name;
+    const SpeechDriver	 *sp_driver;
+    const SpeechVoice	 *sp_voice;
     
-    driver_name = 
-	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_driver)->entry));    
-    driver_voice = 	
-    	gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_voice)->entry));    
+    driver_name = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_driver)->entry));    
+    voice_name  = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (spui_speech_voice_absolute.cb_abs_engine_voice)->entry));    
 
     if (!strcmp (driver_name, SPEECH_ALL_DRIVERS) ||
-	!strcmp (driver_voice, SPEECH_ALL_VOICES))
+	!strcmp (voice_name, SPEECH_ALL_VOICES)  ||
+	!driver_name || !voice_name)
     {
-	spui_send_msg (_("Invalide driver or voice!"));	
+	gn_show_message (_("Invalid driver or voice!"));	
 	return;
     }
-        
+    
+    sp_driver = spconf_driver_list_get_driver_from_name_descr (sp_speech_driver_list,
+							    driver_name);
+    sp_voice  = spconf_driver_list_get_speech_voice (sp_speech_driver_list,
+						     sp_driver->driver_name, 
+						     voice_name);
+    if (!sp_voice  ||
+	!sp_driver)
+	return;
+	
     for (tmp = sp_speakers_settings; tmp ; tmp = tmp->next)
     {
 	SpeakerSettings *sp_set = (SpeakerSettings *)tmp->data;
-	if (strcmp (sp_set->driver_name, driver_name))
+	
+	if (strcmp (sp_set->driver_name_descr, driver_name))
 	{
 	    g_free (sp_set->driver_name);
-	    sp_set->driver_name = g_strdup (driver_name);
+	    g_free (sp_set->driver_name_descr);
+	    sp_set->driver_name_descr = g_strdup (sp_driver->driver_name_descr);
+	    sp_set->driver_name       = g_strdup (sp_driver->driver_name);
 	}
-	if (strcmp (sp_set->driver_voice, driver_voice))
+	
+	if (strcmp (sp_set->voice_name_descr, voice_name))
 	{
-	    g_free (sp_set->driver_voice);
-	    sp_set->driver_voice = g_strdup (driver_voice);
+	    g_free (sp_set->voice_name);
+	    g_free (sp_set->voice_name_descr);
+	    sp_set->voice_name 		= g_strdup (sp_voice->voice_name);
+	    sp_set->voice_name_descr 	= g_strdup (sp_voice->voice_name_descr);
 	}
     }
 
     spconf_speaker_settings_list_save (sp_speakers_settings);
-
+    spui_voice_absolute_properties_sensitivity_set (TRUE);
 }
 
 static void
@@ -1096,36 +1215,67 @@ static void
 spui_voice_absolute_volume_active (GtkWidget *widget,
 				   gpointer  user_data)
 {
+    GtkWidget *label;
     spui_speech_voice_absolute.abs_volume_sens =
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.sp_abs_volume), 
 			      spui_speech_voice_absolute.abs_volume_sens);
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.hs_abs_volume), 
 			      spui_speech_voice_absolute.abs_volume_sens);
+			      
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_volume),
+				"abs_volume");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_volume_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_volume),
+				"abs_volume_perc");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_volume_sens);
+
 }
 
 static void
 spui_voice_absolute_rate_active (GtkWidget *widget,
 				 gpointer  user_data)
 {
+    GtkWidget *label;
     spui_speech_voice_absolute.abs_rate_sens =
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.sp_abs_rate), 
 			      spui_speech_voice_absolute.abs_rate_sens);
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.hs_abs_rate), 
 			      spui_speech_voice_absolute.abs_rate_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_rate),
+				"abs_rate");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_rate_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_rate),
+				"abs_rate_word_min");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_rate_sens);
+
 }
 
 static void
 spui_voice_absolute_pitch_active (GtkWidget *widget,
 				  gpointer  user_data)
 {
+    GtkWidget *label;
     spui_speech_voice_absolute.abs_pitch_sens =
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.sp_abs_pitch), 
 			      spui_speech_voice_absolute.abs_pitch_sens);
     gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice_absolute.hs_abs_pitch), 
 			      spui_speech_voice_absolute.abs_pitch_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_pitch),
+				"abs_pitch");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_pitch_sens);
+    label = (GtkWidget*)g_object_get_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_pitch),
+				"abs_pitch_hz");
+    gtk_widget_set_sensitive (GTK_WIDGET (label), 
+			      spui_speech_voice_absolute.abs_pitch_sens);
+
 }
 
 static void
@@ -1194,6 +1344,15 @@ spui_voice_absolute_sp_value_changed (GtkWidget *widget,
 
 }
 
+
+static gboolean
+spui_voice_absolute_sp_value_output (GtkWidget *widget,
+				    	   gpointer   user_data)
+{
+    spui_voice_absolute_sp_value_changed (widget, user_data);
+    return FALSE;
+}
+
 static void
 spui_voice_absolute_response (GtkDialog *dialog,
 			      gint       response_id,
@@ -1201,11 +1360,10 @@ spui_voice_absolute_response (GtkDialog *dialog,
 {
     switch (response_id)
     {
+	case GTK_RESPONSE_CLOSE:
+	break;
 	case GTK_RESPONSE_OK: 
         break;
-	case GTK_RESPONSE_CANCEL:
-	    spconf_speaker_settings_list_save (sp_speakers_settings_abs_backup);
-        break;
 	case GTK_RESPONSE_HELP: 
 	    gn_load_help ("gnopernicus-speech-settings");
 	    return;
@@ -1213,8 +1371,6 @@ spui_voice_absolute_response (GtkDialog *dialog,
 	default:
         break;
     }
-    sp_speakers_settings_abs_backup =
-	    spconf_speaker_settings_list_free (sp_speakers_settings_abs_backup);
     gtk_widget_hide ((GtkWidget*)dialog);
     sp_abs_changes_activable = FALSE;
 }
@@ -1237,6 +1393,19 @@ spui_voice_absolute_handlers_set  (GladeXML *xml)
     spui_speech_voice_absolute.ck_abs_rate 	   	= glade_xml_get_widget (xml, "ck_abs_rate_active");
     spui_speech_voice_absolute.ck_abs_volume 	   	= glade_xml_get_widget (xml, "ck_abs_volume_active");    
 
+    g_object_set_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_volume), 
+			"abs_volume", glade_xml_get_widget (xml, "lb_abs_volume"));
+    g_object_set_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_volume), 
+			"abs_volume_perc", glade_xml_get_widget (xml, "lb_abs_volume_perc"));
+    g_object_set_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_rate), 
+			"abs_rate", glade_xml_get_widget (xml, "lb_abs_rate"));
+    g_object_set_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_rate), 
+			"abs_rate_word_min", glade_xml_get_widget (xml, "lb_abs_rate_word_min"));
+    g_object_set_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_pitch), 
+			"abs_pitch", glade_xml_get_widget (xml, "lb_abs_pitch"));
+    g_object_set_data (G_OBJECT (spui_speech_voice_absolute.ck_abs_pitch), 
+			"abs_pitch_hz", glade_xml_get_widget (xml, "lb_abs_pitch_hz"));
+
     g_signal_connect (spui_speech_voice_absolute.w_voice_setting_absolute_values, "response",
 		      G_CALLBACK (spui_voice_absolute_response), NULL);
     g_signal_connect (spui_speech_voice_absolute.w_voice_setting_absolute_values, "delete_event",
@@ -1271,22 +1440,27 @@ spui_voice_absolute_handlers_set  (GladeXML *xml)
     glade_xml_signal_connect_data (xml, "on_sp_abs_volume_changed",			
 			    GTK_SIGNAL_FUNC (spui_voice_absolute_sp_value_changed), 
 			    (gpointer)spui_speech_voice_absolute.hs_abs_volume);
+    glade_xml_signal_connect_data (xml, "on_sp_abs_volume_output",			
+			    GTK_SIGNAL_FUNC (spui_voice_absolute_sp_value_output), 
+			    (gpointer)spui_speech_voice_absolute.hs_abs_volume);
+
     glade_xml_signal_connect_data (xml, "on_sp_abs_rate_changed",			
 			    GTK_SIGNAL_FUNC (spui_voice_absolute_sp_value_changed), 
 			    (gpointer)spui_speech_voice_absolute.hs_abs_rate);
+    glade_xml_signal_connect_data (xml, "on_sp_abs_rate_output",			
+			    GTK_SIGNAL_FUNC (spui_voice_absolute_sp_value_output), 
+			    (gpointer)spui_speech_voice_absolute.hs_abs_rate);
     glade_xml_signal_connect_data (xml, "on_sp_abs_pitch_changed",			
 			    GTK_SIGNAL_FUNC (spui_voice_absolute_sp_value_changed),
 			    (gpointer)spui_speech_voice_absolute.hs_abs_pitch);
+    glade_xml_signal_connect_data (xml, "on_sp_abs_pitch_output",			
+			    GTK_SIGNAL_FUNC (spui_voice_absolute_sp_value_output), 
+			    (gpointer)spui_speech_voice_absolute.hs_abs_pitch);
 }
 
 static gboolean 
 spui_voice_absolute_load (GtkWidget *parent_window)
 {         
-    sru_assert (!sp_speakers_settings_abs_backup);
-
-    sp_speakers_settings_abs_backup =
-	    spconf_speaker_settings_list_clone (sp_speakers_settings);
-
     if (!spui_speech_voice_absolute.w_voice_setting_absolute_values)
     {
 	GladeXML *xml;
@@ -1310,6 +1484,9 @@ spui_voice_absolute_load (GtkWidget *parent_window)
     return TRUE;
 }
 
+
+
+
 /******************************************************************************/
 static gboolean
 spui_voice_settings_button_add_clicked (GtkWidget *widget,
@@ -1369,9 +1546,12 @@ spui_voice_settings_button_modify_clicked (GtkWidget *widget,
 
     if (!GTK_IS_LIST_STORE (model))
 	return FALSE;
+	
     if (!gtk_tree_selection_get_selected (selection, NULL, &sp_curr_iter))
+    {
+    	gn_show_message (_("No selected voice to modify!"));
 	return FALSE;
-
+    }
     /* load UI and show it*/
     spui_voice_add_modify_load (spui_speech_voice.w_voice_setting);
     
@@ -1425,8 +1605,12 @@ spui_voice_settings_button_remove_clicked (GtkWidget *widget,
 
     if (!GTK_IS_LIST_STORE (model))
 	return;
+	
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+    	gn_show_message (_("No selected voice to remove!"));
 	return;
+    }
     
     gtk_tree_model_get (model, 	&iter,
 			VOICE_GNOPERNICUS_SPEAKER, 
@@ -1494,8 +1678,12 @@ spui_voice_settings_test_clicked (GtkWidget *widget,
 
     if (!GTK_IS_LIST_STORE (model))
 	return;
+	
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+    	gn_show_message (_("No selected voice to test!"));
 	return;
+    }
     
     gtk_tree_model_get (model, 	&iter,
     			VOICE_GNOPERNICUS_SPEAKER, &gnop_speak,
@@ -1539,6 +1727,18 @@ spui_voice_settings_modify_speaker_values (SpeakerSettings *value)
     sp_changes_activable = old;
 }
 
+static void
+spui_selection_changed (GtkTreeSelection *selection,
+			gpointer  user_data)
+{
+    gboolean 	     sensitive;
+
+    sensitive = gtk_tree_selection_get_selected (GTK_TREE_SELECTION (selection), NULL, NULL);
+    
+    gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice.bt_voice_remove), sensitive);
+    gtk_widget_set_sensitive (GTK_WIDGET (spui_speech_voice.bt_voice_modify), sensitive);
+}
+
 void
 spui_voice_settings_gconf_callback (const gchar *path)
 {
@@ -1568,6 +1768,22 @@ spui_voice_settings_gconf_callback (const gchar *path)
     spui_voice_settings_modify_speaker_values ((SpeakerSettings *)sp_list->data);
 }
 
+static gboolean
+spui_voice_settings_search_equal_func (GtkTreeModel  	*model,
+				    gint 		column,
+				    const gchar		*key,
+				    GtkTreeIter		*iter,
+				    gpointer		search_data)
+{
+    gchar *list_key = NULL;
+    
+    gtk_tree_model_get (model, 	iter,
+    			column, &list_key,
+                    	-1);
+
+    return g_strcasecmp (key, list_key) > 0; 
+}
+
 static void
 spui_voice_settings_add_elem_in_table (GtkListStore *store)
 {
@@ -1659,7 +1875,10 @@ spui_voice_settings_handlers_set  (GladeXML *xml)
     GtkTreeViewColumn 	*column;
 
     spui_speech_voice.w_voice_setting = glade_xml_get_widget (xml, "w_voice_setting");
-    spui_speech_voice.tv_voice_table = glade_xml_get_widget (xml, "tv_voice_table");
+    spui_speech_voice.tv_voice_table  = glade_xml_get_widget (xml, "tv_voice_table");
+    spui_speech_voice.bt_voice_modify = glade_xml_get_widget (xml,"bt_voice_modify");	
+    spui_speech_voice.bt_voice_remove =	glade_xml_get_widget (xml,"bt_voice_remove");	
+
     
     g_signal_connect (spui_speech_voice.w_voice_setting, "response",
 		      G_CALLBACK (spui_voice_settings_response), NULL);
@@ -1687,9 +1906,12 @@ spui_voice_settings_handlers_set  (GladeXML *xml)
 			     GTK_SIGNAL_FUNC (spui_voice_settings_test_clicked));
 
     model = spui_voice_settings_create_model ();
-                
+                    
     gtk_tree_view_set_model (GTK_TREE_VIEW (spui_speech_voice.tv_voice_table), model);
-
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (spui_speech_voice.tv_voice_table));
+    
+    g_signal_connect (selection, "changed",
+                      G_CALLBACK (spui_selection_changed), NULL);
     
     gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), 
 					  VOICE_GNOPERNICUS_SPEAKER, 
@@ -1752,9 +1974,21 @@ spui_voice_settings_handlers_set  (GladeXML *xml)
     
     g_object_unref (G_OBJECT (model));
     
-    selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW (spui_speech_voice.tv_voice_table));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);    
 
+    spui_selection_changed (selection, NULL);
+    
+    gtk_tree_view_set_search_column 
+	(GTK_TREE_VIEW (spui_speech_voice.tv_voice_table), 
+	VOICE_GNOPERNICUS_SPEAKER);    
+    gtk_tree_view_set_search_equal_func 
+	(GTK_TREE_VIEW (spui_speech_voice.tv_voice_table),
+	spui_voice_settings_search_equal_func,
+	NULL, NULL);
+    gtk_tree_view_set_enable_search 
+	(GTK_TREE_VIEW (spui_speech_voice.tv_voice_table), 
+	TRUE);
+
 }
 
 gboolean 
