@@ -36,7 +36,7 @@
 
 #include <X11/XKBlib.h>
 
-#undef  USE_ALL_WINDOWS
+#define  USE_ALL_WINDOWS
 
 /* Solaris specific variable types. */
 #ifdef __sun
@@ -69,17 +69,7 @@ enum
 
 static gint 			ke_log_flags;
 
-#define KE_INETERSTED_MODIFIERS_NUMBER 6
-enum
-{
-    KB_Shift_Left,
-    KB_Shift_Right,
-    KB_Control_Left,
-    KB_Control_Right,
-    KB_Alt_Left,
-    KB_Alt_Right
-};
-
+#define KE_INTERESTED_MODIFIERS_NUMBER 6
 struct
 {
     gint  	modifier;
@@ -87,7 +77,7 @@ struct
     KeyCode     keyCode;
     u_int32_t   byteIndex;
     u_int32_t   bitIndex;
-} KeModifierMap [KE_INETERSTED_MODIFIERS_NUMBER] =
+} KeModifierMap [KE_INTERESTED_MODIFIERS_NUMBER] =
 {
     {SPI_KEYMASK_SHIFT, 	XK_Shift_L, 	0, 0, 0},
     {SPI_KEYMASK_SHIFT, 	XK_Shift_R, 	0, 0, 0},
@@ -161,8 +151,8 @@ KeyTransTable key_trans_table[]=
 **/
 typedef enum
 {
-	KE_LAYER_STATE_IDLE,
-	KE_LAYER_STATE_LEVEL
+    KE_LAYER_STATE_IDLE,
+    KE_LAYER_STATE_LEVEL
 } KeyboardLayerState;
 
 /**
@@ -190,6 +180,7 @@ typedef struct
     guint modifiers;
 }KEKeyEvent;
 
+static gboolean	ke_user_key_list_unregister_events (void);
 
 /**
  * User key.
@@ -299,9 +290,12 @@ ke_get_log_flag (void)
     const  gchar *log  = NULL;
     
     log = g_getenv ("GNOPERNICUS_LOG");
+    
     if (!log)
 	log = "";
+	
     ke_log_flags = KE_LOG_UNKNOWN;
+    
     if (strstr (log, "at-spi"))
 	ke_log_flags |= KE_LOG_AT_SPI;
     if (strstr (log, "gnopernicus"))
@@ -335,6 +329,7 @@ static void
 ke_srhotkey_data_destructor(gpointer data)
 {
     SRHotkeyData *srhotkey_data = (SRHotkeyData *) data;
+    
     if (srhotkey_data != NULL)
     {
 	g_free (srhotkey_data->keystring);
@@ -355,6 +350,7 @@ static gchar*
 ke_return_keystring (gint keyID)
 {
     gchar *rv = NULL;
+    
     switch (keyID)
     {
         case XK_Alt_L:	rv = g_strdup("Alt_L");break;
@@ -376,7 +372,9 @@ ke_return_keystring (gint keyID)
         default:
     	    rv = NULL;
     }
+    
     sru_debug ("Return keystring:%s",rv);
+    
     return rv;
 }
 
@@ -409,7 +407,7 @@ ke_call_srkey_cb (gulong keyID,
 	srhotkey_data->keyID = keyID;
 	srhotkey_data->modifiers = modifiers;
 	
-	if ( keyID > 255 )
+	if ( keyID > 255 ) /* 255 max printable ASCII code */
 	{
 	    if (!keystring || strlen (keystring) == 0)
 		srhotkey_data->keystring = ke_return_keystring (keyID);
@@ -515,74 +513,58 @@ ke_report_keyecho_event (const AccessibleKeystroke *key, gpointer user_data)
  *
  * Return a layer code.
  *
- * return:layer code.
+ * return:layer code or -1 if not a numpad keys.
 **/
 static gint
 ke_return_key (glong keyID)
 {
     gint layer_code = -1;
+    
     switch (keyID)
     {
-	case XK_KP_0:layer_code = 0; break;
-	case XK_KP_1:layer_code = 1; break;
-	case XK_KP_2:layer_code = 2; break;
-	case XK_KP_3:layer_code = 3; break;
-	case XK_KP_4:layer_code = 4; break;
-	case XK_KP_5:layer_code = 5; break;
-	case XK_KP_6:layer_code = 6; break;
-	case XK_KP_7:layer_code = 7; break;
-	case XK_KP_8:layer_code = 8; break;
-	case XK_KP_9:layer_code = 9; break;
-	case XK_KP_Decimal:layer_code = 10; break;
-	case XK_KP_Enter:layer_code = 11; break;
-	case XK_KP_Add:layer_code = 12; break;
-	case XK_KP_Subtract:layer_code = 13; break;
-	case XK_KP_Multiply:layer_code = 14; break;
-	case XK_KP_Divide:layer_code = 15; break;
+	case XK_KP_0:		layer_code = 0; break;
+	case XK_KP_1:		layer_code = 1; break;
+	case XK_KP_2:		layer_code = 2; break;
+	case XK_KP_3:		layer_code = 3; break;
+	case XK_KP_4:		layer_code = 4; break;
+	case XK_KP_5:		layer_code = 5; break;
+	case XK_KP_6:		layer_code = 6; break;
+	case XK_KP_7:		layer_code = 7; break;
+	case XK_KP_8:		layer_code = 8; break;
+	case XK_KP_9:		layer_code = 9; break;
+	case XK_KP_Decimal:	layer_code = 10; break;
+	case XK_KP_Enter:	layer_code = 11; break;
+	case XK_KP_Add:		layer_code = 12; break;
+	case XK_KP_Subtract:	layer_code = 13; break;
+	case XK_KP_Multiply:	layer_code = 14; break;
+	case XK_KP_Divide:	layer_code = 15; break;
+#ifdef __sun
+	/*
+	 * The KeySyms returned by Solaris Xserver are incorrect.
+	 * The values below are those returned on Solaris.
+	 */
+	case XK_KP_Insert:	layer_code = 0; break;
+	case XK_F33:		layer_code = 1; break;
+	case XK_Down:		layer_code = 2; break;
+	case XK_F35:		layer_code = 3; break;
+	case XK_Left:		layer_code = 4; break;
+	case XK_F31:		layer_code = 5; break;
+	case XK_Right:		layer_code = 6; break;
+	case XK_F27:		layer_code = 7; break;
+	case XK_Up:		layer_code = 8; break;
+	case XK_F29:		layer_code = 9; break;
+	case XK_KP_Delete:	layer_code = 10; break;
+	case XK_F24:		layer_code = 13; break;
+	case XK_F26:		layer_code = 14; break;
+	case XK_F25:		layer_code = 15; break;
+#endif
 	default:
 	break;
     }		
+    
     return layer_code;
 }
 
-/**
- *
- * ke_check_is_keypad_key
- *
- * @keyID:
- *
- * CHeck if a key is from numpad.
- *
- * return: TRUE if is a numpad key.
-**/
-static gboolean
-ke_check_is_keypad_key (glong keyID)
-{
-    switch (keyID)
-    {
-	case XK_KP_0:
-	case XK_KP_1:
-	case XK_KP_2:
-	case XK_KP_3:
-	case XK_KP_4:
-	case XK_KP_5:
-	case XK_KP_6:
-	case XK_KP_7:
-	case XK_KP_8:
-	case XK_KP_9:
-	case XK_KP_Divide:
-	case XK_KP_Multiply:
-	case XK_KP_Add:
-	case XK_KP_Subtract:
-	case XK_KP_Decimal:
-	case XK_KP_Enter:
-	    return TRUE;
-	break;
-	default:
-	break;
-    }
-    return FALSE;
-}
 
 /**
  *
@@ -605,9 +587,13 @@ ke_call_keyboard_layer_cb (const gchar *buf,
     
     sru_return_if_fail (buf);
     sru_return_if_fail (strlen(buf) != 0);
+    
+    /* create SREvent structure */
     evnt = sre_new ();
+    
     sru_return_if_fail (evnt);
 
+    /* fill the event structure */
     evnt->data = g_strdup(buf);
     evnt->type = event_type;
     evnt->data_destructor = g_free;
@@ -618,6 +604,7 @@ ke_call_keyboard_layer_cb (const gchar *buf,
     {
     	ke_log_gnopernicus (evnt);
 
+	/* call event sink from srcore */
         if (ke_keyboard_event_sink_cb != NULL) 
 	    ke_keyboard_event_sink_cb (evnt, flag);
     }
@@ -640,6 +627,7 @@ ke_call_keyboard_layer_cb (const gchar *buf,
 static gboolean
 ke_press_wait_function (gpointer user_data)
 {
+    /* check if other key is pressed then 0 */
     if (ke_layer_state != KE_LAYER_STATE_IDLE)
     {
 	gchar *cmd;
@@ -647,7 +635,8 @@ ke_press_wait_function (gpointer user_data)
 	ke_layer_state = KE_LAYER_STATE_IDLE;
 				
 	cmd = g_strdup_printf("L%02d", ke_layer_level);				
-	
+
+	/* send the time out event */	
 	ke_call_keyboard_layer_cb (cmd, 
 				    KE_LAYER_TIME_OUT, 
 				    SR_EVENT_COMMAND_LAYER_CHANGED);
@@ -656,6 +645,7 @@ ke_press_wait_function (gpointer user_data)
 
 	g_free (cmd);
     }
+    
     return FALSE;
 }
 
@@ -675,6 +665,7 @@ ke_report_layer_key_event (const AccessibleKeystroke *key, gpointer user_data)
 {
     static gboolean layer_busy = FALSE;    
     static GQueue  *layer_queue = NULL;
+    static gint	    layer_timeout_id = 0;
     KEKeyEvent     *ev;
     
     sru_debug ("(Layer)Received key event:\n\tsym %ld (%c)\n\tmods %x\n\tcode %d\n\ttime %ld\n\tkeystring \"%s\"\n\ttype %d (press = %d, release = %d)",
@@ -684,31 +675,37 @@ ke_report_layer_key_event (const AccessibleKeystroke *key, gpointer user_data)
 	      (long int) key->timestamp,
 	      key->keystring,
 	      (gint) key->type, (gint)SPI_KEY_PRESSED, (gint)SPI_KEY_RELEASED );   
-    
+
     ke_log_at_spi (key);	          
     
+    /* filter the release events */
     if (key->type == SPI_KEY_RELEASED)
     {
 	sru_debug ("keyrelease received");
 	return TRUE;
     }
 
+    /* create event structure */
     ev = (KEKeyEvent *) g_new0 (KEKeyEvent, 1);
     
     sru_return_val_if_fail (ev, FALSE);
     
+    /* create the event queue */
     if (!layer_queue)
 	layer_queue = g_queue_new ();
 
+    /* fill the event structure */
     ev->keyID     = key->keyID;
     ev->modifiers = key->modifiers;
     ev->keycode	  = key->keycode;
     ev->keystring = NULL;
     
+    /* push event in queue */
     g_queue_push_head (layer_queue, ev);
 
     sru_debug ("layer_busy:%d", layer_busy);
     
+    /* check for reentrancy */
     sru_return_val_if_fail (!layer_busy, TRUE);
     
     layer_busy = TRUE;
@@ -716,62 +713,73 @@ ke_report_layer_key_event (const AccessibleKeystroke *key, gpointer user_data)
     while (!g_queue_is_empty (layer_queue))
     {
 	KEKeyEvent *key1;
-		
+	gint	   level = -1;
+	
+	/* pop event from queue */
 	key1 = (KEKeyEvent *) g_queue_pop_tail (layer_queue);
 		
-	if (ke_check_is_keypad_key (key1->keyID))
+	/* check if it is a valid numpad key and return the level */
+	if ((level = ke_return_key (key1->keyID)) != -1)
 	{
 	    switch (ke_layer_state)
 	    {
 		case KE_LAYER_STATE_IDLE:
 		{
-			if (key1->keyID == XK_KP_0)
-			{
-			    sru_debug ("Layer change key pressed");
-			    ke_layer_state = KE_LAYER_STATE_LEVEL;
-			    g_timeout_add ( KE_TIME_WAIT , 
-					    ke_press_wait_function , 
-					    NULL ) ;
-			}
-			else
-			{
-			    gchar *cmd;
+		    if (level == 0)
+		    {			    
+			sru_debug ("Layer change key pressed");
 			    
-			    ke_layer_state = KE_LAYER_STATE_IDLE;
+			ke_layer_state = KE_LAYER_STATE_LEVEL;
 			    
-			    cmd = g_strdup_printf ("L%02dK%02d", 
-						    ke_layer_level, 
-						    ke_return_key (key1->keyID));				
+			/* wait to press a numpad key that to change the layer */
+			layer_timeout_id = g_timeout_add (KE_TIME_WAIT , 
+							  ke_press_wait_function , 
+							  NULL);
+		    }
+		    else
+		    {
+			gchar *cmd = NULL;
+			    
+			ke_layer_state = KE_LAYER_STATE_IDLE;
+
+			/* create a command */
+			cmd = g_strdup_printf ("L%02dK%02d", 
+						ke_layer_level, 
+						level);				
 						    
-			    ke_call_keyboard_layer_cb (cmd, 0, SR_EVENT_COMMAND_LAYER);
-			    
-			    sru_debug ("Command code %s",cmd);				
-			    
-			    g_free (cmd);
-			}
+			ke_call_keyboard_layer_cb (cmd, 0, SR_EVENT_COMMAND_LAYER);
+				    
+			sru_debug ("Command code %s",cmd);				
+			
+			g_free (cmd);
+		    }
 		    break;
 		}
 		case KE_LAYER_STATE_LEVEL:
 		{
-			if (ke_check_is_keypad_key (key1->keyID))
-			{
-			    gchar *cmd = NULL;
+		    gchar *cmd = NULL;
+		    
+		    if (layer_timeout_id)
+			g_source_remove (layer_timeout_id);
+		    
+		    layer_timeout_id = 0;
+		    
+		    ke_layer_level   = level;
 			    
-			    ke_layer_level = ke_return_key (key1->keyID);
-			    
-			    cmd = g_strdup_printf ("L%02d", ke_layer_level);
+		    /* create a layer change command */
+		    cmd = g_strdup_printf ("L%02d", ke_layer_level);
 					
-			    ke_call_keyboard_layer_cb ( cmd, 
-							KE_LAYER_CHANGED, 
-							SR_EVENT_COMMAND_LAYER_CHANGED);
+		    ke_call_keyboard_layer_cb ( cmd, 
+						KE_LAYER_CHANGED, 
+						SR_EVENT_COMMAND_LAYER_CHANGED);
 			     
-			    sru_debug ("Command:%s",cmd);
+		    sru_debug ("Command:%s",cmd);
 			    
-			    g_free (cmd);
-			}
+		    g_free (cmd);
 			    
-			ke_layer_state = KE_LAYER_STATE_IDLE;
-			break;
+		    ke_layer_state = KE_LAYER_STATE_IDLE;
+			
+		    break;
 		}
 		default:
 		    ke_layer_state = KE_LAYER_STATE_IDLE;
@@ -803,7 +811,9 @@ static KeySetItem*
 ke_key_set_item_new (void)
 {
     KeySetItem *item = NULL;
+    
     item = (KeySetItem *) g_new0 (KeySetItem, 1);
+    
     return item;    
 }
 
@@ -882,65 +892,6 @@ ke_translate_key (const gchar *key)
     return g_strdup (key);
 }
 
-/**
- * ke_return_modifier_mask
- *
- * @data:
- * @modifiers:
- * @key:
- *
- * Return modifier and key from data.
- *
- * return: FALSE at failed and invalid string.
-**/
-static gboolean
-ke_return_modifier_and_key (const gchar *data,
-			    unsigned long *modifiers,
-			    gchar  **key)
-{
-    gchar *delimit;
-    gchar *iter;
-    unsigned long rv = 0;
-    
-    *modifiers = SPI_KEYMASK_UNMODIFIED;
-    *key = NULL;
-    
-    sru_return_val_if_fail (data, FALSE);
-    
-    delimit = g_strrstr (data, "-");
-
-    if (!delimit)
-    {
-	*key = g_strdup (data);
-	sru_debug ("No delimiter in key combination.");
-	return TRUE;
-    }
-    
-    iter = (gchar*)data;
-    
-    while (*iter != '-')
-    {
-	switch (*iter)
-	{
-	    case 'A':
-		rv = rv | SPI_KEYMASK_ALT;
-	    break;
-	    case 'C':
-		rv = rv | SPI_KEYMASK_CONTROL;
-	    break;
-	    case 'S':
-		rv = rv | SPI_KEYMASK_SHIFT;
-	    break;
-	    default:
-		return FALSE;
-	    break;
-	}
-	iter++;
-    }
-    *modifiers = rv;
-    *key       = g_strdup (delimit + 1);
-    return TRUE;
-}
 /**
  * ke_string_to_keycode
  *
@@ -953,14 +904,17 @@ ke_return_modifier_and_key (const gchar *data,
 static gchar*
 ke_string_to_keysym (const gchar *string, gboolean modifier)
 {
-    gchar   *keysym = NULL;
-    gchar *tmp_str;
+    gchar   *keysym  = NULL;
+    gchar   *tmp_str = NULL;
     long     lkeysym;
+
+    modifier = modifier & SPI_KEYMASK_SHIFT;
     
     if (g_utf8_strlen (string, -1) == 1)
     {
         gunichar uchar = g_utf8_get_char (string);
 	
+	/* check if shift modifier is active */
         if (modifier)
 	{
 	    if (g_unichar_isdigit (uchar))
@@ -980,7 +934,9 @@ ke_string_to_keysym (const gchar *string, gboolean modifier)
 	else
 	    tmp_str = g_strdup (string);
     }
+    
     lkeysym  = XStringToKeysym (tmp_str);
+    
     g_free (tmp_str);
      
     if (g_unichar_validate ((guint32)lkeysym))
@@ -1010,177 +966,20 @@ ke_keysym_to_string (glong keysym, gint modifier)
     gchar *keystring = NULL;
     gchar *tmp = NULL;
 
+    /* if special character code [0-32] */
     if (keysym <= Z_CHAR_POSITION)
 	return g_strdup_printf ("%c", ('A' + (gint)keysym - 1));
 	    	
+    /* return a keystring from keysym */
     tmp = XKeysymToString (keysym);
 
+    /* translate a key */
     keystring = ke_translate_key_revers (tmp, (modifier & SPI_KEYMASK_SHIFT));
 
 /*    g_free (tmp);*/ /*FIXME: tmp could be free or not? */
     return keystring;
 }
 
-/**
- * ke_register_user_key_list:
- *
- * @list: user keylist.
- *
- * Register a user keylist events.
- *
-**/
-static void
-ke_register_user_key_list (GSList *list)
-{
-    GSList *elem = NULL;
-    elem = list;
-    
-    while (elem)
-    {
-        unsigned long modifiers;
-	gchar *key;
-	
-	if (ke_return_modifier_and_key ((gchar*)elem->data, &modifiers, &key))
-	{
-	    KeySetItem 		*ksi = NULL;
-	    AccessibleKeySet 	*keyset = NULL;
-	    gchar		*keysym = NULL;
-
-	    if (!key) 
-	    {
-	    	elem = elem->next;
-		continue;
-	    }
-	    
-	    ksi = ke_key_set_item_new ();
-	    
-	    if (!ksi) 
-	    {
-		g_free (key);
-		elem = elem->next;
-		continue;
-	    }
-	    
-	    sru_debug ("Modifier %s combination:%ld-%s",(gchar*)elem->data,modifiers,key);
-
-	    keysym   = ke_string_to_keysym (key, modifiers & SPI_KEYMASK_SHIFT);
-	    keyset   = SPI_createAccessibleKeySet (1, (const char*)keysym, NULL, NULL);
-	    g_free (keysym);
-	    
-	    ksi->keyset   = keyset;
-	    ksi->modifier = modifiers;
-	    
-	    accessible_key_set_list = g_slist_append (accessible_key_set_list, ksi);
-	
-	    if (ksi->keyset)
-	    {						      
-	        SPIBoolean retval = FALSE;
-
-		retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
-				    			(AccessibleKeySet *) ksi->keyset,
-				    			ksi->modifier,
-				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-		ke_print_register_return_value (retval, "");
-		retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
-				    			(AccessibleKeySet *) ksi->keyset,
-				    			ksi->modifier | SPI_KEYMASK_BUTTON1,
-				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-		ke_print_register_return_value (retval, "");
-		retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
-				    			(AccessibleKeySet *) ksi->keyset,
-				    			ksi->modifier | SPI_KEYMASK_BUTTON2,
-				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-		ke_print_register_return_value (retval, "");
-		retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
-				    			(AccessibleKeySet *) ksi->keyset,
-				    			ksi->modifier | SPI_KEYMASK_BUTTON3,
-				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-		ke_print_register_return_value (retval, "");
-	    }	    
-	}
-	g_free (key);
-	elem = elem->next;
-    }
-}
-
-/**
- * ke_unregister_user_key_list:
- *
- * Unregister user keys.
- *
- * return: TRUE if success.
-**/
-static gboolean
-ke_unregister_user_key_list (void)
-{
-    GSList *elem = NULL;
-
-    if (!accessible_key_set_list)
-	return FALSE;
-    	
-    elem = accessible_key_set_list;
-    
-    while (elem)
-    {
-	KeySetItem *ksi = elem->data;
-	
-	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
-						   ksi->modifier);
-
-	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
-						   ksi->modifier | 
-						   SPI_KEYMASK_BUTTON1);
-
-	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
-						   ksi->modifier |
-						   SPI_KEYMASK_BUTTON2);
-
-	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
-						   ksi->modifier |
-						   SPI_KEYMASK_BUTTON3);
-	
-	ke_key_set_item_free (ksi);
-	
-	elem->data = NULL;
-	
-	elem = elem->next;
-    }
-    
-    g_slist_free (accessible_key_set_list);
-    
-    accessible_key_set_list = NULL;
-    
-    return TRUE;
-}
-
-/**
- * ke_user_key_list_free:
- *
- * @list: user_key_list
- *
- * Free a user key list.
- *
-**/
-static void
-ke_user_key_list_free (GSList *list)
-{
-    GSList *elem = NULL;    
-
-    sru_return_if_fail (list);
-
-    for (elem = list ; elem ; elem = elem->next)
-	g_free (elem->data);
-    
-    g_slist_free (list);
-    
-    list = NULL;
-}
-
-
 /**
  * ke_return_active_modifiers
  *
@@ -1203,7 +1002,7 @@ ke_return_active_modifiers (void)
     }
     
     /* Return the keyvector */
-    XQueryKeymap(display, keyVector);
+    XQueryKeymap (display, keyVector);
         
     /*Return state of Shifts, Alts, Controls and for the Num Lock */
     for (i = 0 ; i < G_N_ELEMENTS (KeModifierMap) ; i++)
@@ -1217,7 +1016,7 @@ ke_return_active_modifiers (void)
 	}
                                                                                 
 	KeModifierMap[i].byteIndex = (KeModifierMap[i].keyCode/8);
-	KeModifierMap[i].bitIndex = KeModifierMap[i].keyCode & 7;
+	KeModifierMap[i].bitIndex  = KeModifierMap[i].keyCode & 7;
 	
 	if (1 & (keyVector[KeModifierMap[i].byteIndex] >> KeModifierMap[i].bitIndex)) 
 	    rv = rv | KeModifierMap[i].modifier;
@@ -1262,22 +1061,26 @@ ke_report_user_key_event (const AccessibleKeystroke *key, gpointer user_data)
 	return TRUE;
     }
     
-    /* create queue and push in quueue the event */    
+    /* create event struct */    
     ev = (KEKeyEvent *) g_new0 (KEKeyEvent, 1);
     
     sru_return_val_if_fail (ev, FALSE);
 
+    /* create queue if not exist yet */
     if (!user_queue)
 	user_queue = g_queue_new ();
 
+    /* fill event struct */
     ev->keyID     = key->keyID;
     ev->modifiers = key->modifiers;
     ev->keycode	  = key->keycode;
     ev->keystring = NULL;
 
+    /* push event in queue */
     g_queue_push_head (user_queue, ev);
 
     sru_debug ("user_busy:%d", user_busy);
+    /* check for reentrancy */
     sru_return_val_if_fail (!user_busy, TRUE);
 
     user_busy = TRUE;
@@ -1320,6 +1123,178 @@ ke_report_user_key_event (const AccessibleKeystroke *key, gpointer user_data)
     return TRUE;
 }
 
+/**
+ * ke_return_modifier_mask
+ *
+ * @data:
+ * @modifiers:
+ * @key:
+ *
+ * Return modifier and key from data.
+ *
+ * return: FALSE at failed and invalid string.
+**/
+static gboolean
+ke_return_modifier_and_key (const gchar *data,
+			    unsigned long *modifiers,
+			    gchar  **key)
+{
+    gchar *delimit;
+    gchar *iter;
+    unsigned long rv = 0;
+    
+    *modifiers = SPI_KEYMASK_UNMODIFIED;
+    *key = NULL;
+    
+    sru_return_val_if_fail (data, FALSE);
+    
+    delimit = g_strrstr (data, "-");
+
+    if (!delimit)
+    {
+	*key = g_strdup (data);
+	sru_debug ("No delimiter in key combination.");
+	return TRUE;
+    }
+    
+    iter = (gchar*)data;
+    
+    /* get the modifier */
+    while (*iter != '-')
+    {
+	switch (*iter)
+	{
+	    case 'A':
+		rv = rv | SPI_KEYMASK_ALT;
+	    break;
+	    case 'C':
+		rv = rv | SPI_KEYMASK_CONTROL;
+	    break;
+	    case 'S':
+		rv = rv | SPI_KEYMASK_SHIFT;
+	    break;
+	    default:
+		return FALSE;
+	    break;
+	}
+	iter++;
+    }
+    
+    *modifiers = rv;
+    *key       = g_strdup (delimit + 1);
+    
+    return TRUE;
+}
+
+
+static gboolean
+ke_register_keysetitem (KeySetItem *ksi)
+{
+    SPIBoolean retval = FALSE;
+
+    retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
+			    				(AccessibleKeySet *) ksi->keyset,
+				    			ksi->modifier,
+				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+    ke_print_register_return_value (retval, "");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
+				    			(AccessibleKeySet *) ksi->keyset,
+				    			ksi->modifier | SPI_KEYMASK_BUTTON1,
+				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+    ke_print_register_return_value (retval, "");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
+				    			(AccessibleKeySet *) ksi->keyset,
+				    			ksi->modifier | SPI_KEYMASK_BUTTON2,
+				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+    ke_print_register_return_value (retval, "");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
+				    			(AccessibleKeySet *) ksi->keyset,
+				    			ksi->modifier | SPI_KEYMASK_BUTTON3,
+				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+    ke_print_register_return_value (retval, "");
+
+    return TRUE;
+}
+
+
+/**
+ * ke_user_key_list_register_events:
+ *
+ * @list: user keylist.
+ *
+ * Register a user keylist events.
+ *
+**/
+static void
+ke_user_key_list_register_events (GSList *list)
+{
+    GSList *elem = NULL;
+    elem = list;
+    
+    while (elem)
+    {
+        unsigned long modifiers;
+	gchar *key;
+
+	/* return the modifiers	and the key from data */
+	if (ke_return_modifier_and_key ((gchar*)elem->data, &modifiers, &key))
+	{
+	    KeySetItem 		*ksi = NULL;
+	    AccessibleKeySet 	*keyset = NULL;
+	    gchar		*keysym = NULL;
+
+	    /* if no key go to next step */
+	    if (!key) 
+	    {
+	    	elem = elem->next;
+		continue;
+	    }
+	    
+	    /* create key set item */
+	    ksi = ke_key_set_item_new ();
+	    
+	    if (!ksi) 
+	    {
+		g_free (key);
+		elem = elem->next;
+		continue;
+	    }
+	    
+	    sru_debug ("Modifier %s combination:%ld-%s",(gchar*)elem->data,modifiers,key);
+	    
+	    /* return a keysym values from key */
+	    keysym   = ke_string_to_keysym (key, modifiers & SPI_KEYMASK_SHIFT);
+	    
+	    /* create an AccessibleKeySet with a keysym */
+	    keyset   = SPI_createAccessibleKeySet (1, (const char*)keysym, NULL, NULL);
+	    
+	    g_free (keysym);
+	    
+	    /* fill the key set item */
+	    ksi->keyset   = keyset;
+	    ksi->modifier = modifiers;
+	    
+	    /* append keyset item in list */
+	    accessible_key_set_list = g_slist_append (accessible_key_set_list, ksi);
+	
+	    /* register key events */
+	    if (ksi->keyset)
+	    {	
+		ke_register_keysetitem (ksi);
+	    }
+	}
+	
+	g_free (key);
+	
+	elem = elem->next;
+    }
+}
+
+
 static gboolean
 ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 			  AccessibleKeySet 	      *ke_layer_keyset)
@@ -1382,6 +1357,90 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B1|B2|B3|NUMLOCK)");
 #endif
+
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif
+
+    ke_print_register_return_value (retval, "(NUMLOCK)");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_SHIFTLOCK|SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif
+    ke_print_register_return_value (retval, "(SL|NUMLOCK)");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif
+    ke_print_register_return_value (retval, "(S|NUMLOCK)");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_ALT|SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS				
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif
+    ke_print_register_return_value (retval, "(S|NUMLOCK)");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif				      
+    ke_print_register_return_value (retval, "(C|NUMLOCK)");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_ALT|SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif				      
+    ke_print_register_return_value (retval, "(A|S|NUMLOCK)");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif
+    ke_print_register_return_value (retval, "(C|S|NUMLOCK)");
+    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
+				      (AccessibleKeySet *) ke_layer_keyset,
+				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_ALT|SPI_KEYMASK_MOD3,
+				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+#ifdef USE_ALL_WINDOWS
+				      SPI_KEYLISTENER_ALL_WINDOWS);
+#else
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+#endif
+    ke_print_register_return_value (retval, "(C|A|NUMLOCK)");
+    
+
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
 				      SPI_KEYMASK_MOD2,
@@ -1466,6 +1525,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     
     return retval;
 }
+
 static gboolean
 ke_keyecho_register_events (AccessibleKeystrokeListener *ke_key_listener,
 			    AccessibleKeySet 	        *ke_key_keyset)
@@ -1477,6 +1537,22 @@ ke_keyecho_register_events (AccessibleKeystrokeListener *ke_key_listener,
 	SHIFT_L, SHIFT_R, Control_L, Control_R, Up, Down, Left, Right, 
 	PageUp, PageDown, Home, End, CapsLock, NumLock,
     */
+    retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
+				      (AccessibleKeySet *) ke_key_keyset,
+				      SPI_KEYMASK_BUTTON1,
+				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+    retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
+				      (AccessibleKeySet *) ke_key_keyset,
+				      SPI_KEYMASK_BUTTON2,
+				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+    retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
+				      (AccessibleKeySet *) ke_key_keyset,
+				      SPI_KEYMASK_BUTTON3,
+				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
+				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
+				      
     retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
 				      (AccessibleKeySet *) ke_key_keyset,
 				      SPI_KEYMASK_UNMODIFIED,
@@ -1631,6 +1707,30 @@ ke_get_config_settings (GSList **list)
     return TRUE;
 }
 
+/**
+ * ke_user_key_list_free:
+ *
+ * @list: user_key_list
+ *
+ * Free a user key list.
+ *
+**/
+static void
+ke_user_key_list_free (GSList *list)
+{
+    GSList *elem = NULL;    
+
+    sru_return_if_fail (list);
+
+    for (elem = list ; elem ; elem = elem->next)
+	g_free (elem->data);
+    
+    g_slist_free (list);
+    
+    list = NULL;
+}
+
+
 /**
  * ke_config_changed:
  *
@@ -1641,14 +1741,15 @@ ke_config_changed (void)
 {
     sru_debug ("ke_config_changed invoked.");
     
-    ke_unregister_user_key_list ();  
+    ke_user_key_list_unregister_events ();  
+    
     ke_user_key_list_free (ke_user_key_list);
     ke_user_key_list_free (reg_list);
     ke_user_key_list = NULL;
     
     if (ke_get_config_settings (&ke_user_key_list))
     {
-	ke_register_user_key_list   (ke_user_key_list);
+	ke_user_key_list_register_events (ke_user_key_list);
     }
 }
 
@@ -1671,13 +1772,22 @@ ke_init (KeyboardEchoCB kecb)
   
     sru_return_val_if_fail (ke_keyboard_status == KE_IDLE, FALSE);
     sru_return_val_if_fail (kecb, FALSE);
-    
+
+    /*
+	Get log flag from enviroment
+    */    
     ke_get_log_flag ();
     
     sru_debug ("ke_init...");
     
+    /*
+	Attribute keyboard event sink function from srcore.
+    */
     ke_keyboard_event_sink_cb = kecb;
 
+    /*
+	User keys get from gconf.
+    */
     ke_user_key_list = NULL;
     ke_get_config_settings (&ke_user_key_list);
       
@@ -1694,6 +1804,7 @@ ke_init (KeyboardEchoCB kecb)
     ke_layer_keycodes = ke_get_keypad_keycodes ();
     ke_layer_keyset   = SPI_createAccessibleKeySet (COUNT_OF_ITEM_IN_LAYER_KEYSET, NULL, ke_layer_keycodes, NULL);
     g_free (ke_layer_keycodes);
+
     /*
 	Keyboard echo.
     */
@@ -1705,7 +1816,7 @@ ke_init (KeyboardEchoCB kecb)
 	User key registering.
     */  
     if (ke_user_key_list)
-	ke_register_user_key_list (ke_user_key_list);
+	ke_user_key_list_register_events (ke_user_key_list);
 
     /*
 	Layer key registering.
@@ -1726,28 +1837,86 @@ ke_init (KeyboardEchoCB kecb)
 }
 
 /**
- * ke_terminate
+ * ke_user_key_list_unregister_events:
  *
- * Deregister key listener(s)
+ * Unregister user keys.
  *
+ * return: TRUE if success.
 **/
-void
-ke_terminate (void)
+static gboolean
+ke_user_key_list_unregister_events (void)
 {
-    sru_return_if_fail (ke_keyboard_status != KE_IDLE);
-    sru_debug ("ke_terminate...");
+    GSList *elem = NULL;
 
-  /* deregister keylisteners */
-    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_UNMODIFIED );
-    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_MOD2 );
-    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_SHIFTLOCK );
-    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_SHIFTLOCK |
-					    SPI_KEYMASK_MOD2 );
-    AccessibleKeystrokeListener_unref (ke_keyecho_listener);
+    if (!accessible_key_set_list)
+	return FALSE;
+    	
+    elem = accessible_key_set_list;
+    
+    while (elem)
+    {
+	KeySetItem *ksi = elem->data;
+	
+	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
+						   ksi->modifier);
+
+	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
+						   ksi->modifier | 
+						   SPI_KEYMASK_BUTTON1);
+
+	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
+						   ksi->modifier |
+						   SPI_KEYMASK_BUTTON2);
+
+	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
+						   ksi->modifier |
+						   SPI_KEYMASK_BUTTON3);
+	
+	ke_key_set_item_free (ksi);
+	
+	elem->data = NULL;
+	
+	elem = elem->next;
+    }
+    
+    g_slist_free (accessible_key_set_list);
+    
+    accessible_key_set_list = NULL;
+    
+    return TRUE;
+}
+
+static void 
+ke_layer_unregister_events (void)
+{
+#ifndef USE_ALL_WINDOWS
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_BUTTON1 |
+					    SPI_KEYMASK_MOD2);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_BUTTON2 |
+					    SPI_KEYMASK_MOD2);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_BUTTON3 |
+					    SPI_KEYMASK_MOD2);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_BUTTON1 |
+					    SPI_KEYMASK_BUTTON2 |
+					    SPI_KEYMASK_MOD2);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_BUTTON1 |
+					    SPI_KEYMASK_BUTTON3 |
+					    SPI_KEYMASK_MOD2);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_BUTTON2 |
+					    SPI_KEYMASK_BUTTON3 |
+					    SPI_KEYMASK_MOD2);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_BUTTON1 |
+					    SPI_KEYMASK_BUTTON2 |
+					    SPI_KEYMASK_BUTTON3 |
+					    SPI_KEYMASK_MOD2);
+#endif
 
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_MOD2);
@@ -1775,11 +1944,84 @@ ke_terminate (void)
 					    SPI_KEYMASK_MOD2  | 
 					    SPI_KEYMASK_SHIFT |
 					    SPI_KEYMASK_ALT);
-    AccessibleKeystrokeListener_unref (ke_layer_listener);
 
-    ke_unregister_user_key_list ();  
-    ke_user_key_list_free  (ke_user_key_list);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3 | 
+					    SPI_KEYMASK_SHIFTLOCK);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3 | 
+					    SPI_KEYMASK_SHIFT);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3 | 
+					    SPI_KEYMASK_ALT);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3 | 
+					    SPI_KEYMASK_CONTROL);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3  | 
+					    SPI_KEYMASK_SHIFT |
+					    SPI_KEYMASK_CONTROL);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3  | 
+					    SPI_KEYMASK_ALT   |
+					    SPI_KEYMASK_CONTROL);
+    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
+					    SPI_KEYMASK_MOD3  | 
+					    SPI_KEYMASK_SHIFT |
+					    SPI_KEYMASK_ALT);
+}
+
+static void
+ke_keyecho_unregister_events (void)
+{
+    /* deregister keylisteners */
+    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
+					    SPI_KEYMASK_BUTTON1);
+    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
+					    SPI_KEYMASK_BUTTON2);
+    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
+					    SPI_KEYMASK_BUTTON3);
+    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
+					    SPI_KEYMASK_UNMODIFIED );
+    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
+					    SPI_KEYMASK_MOD2 );
+    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
+					    SPI_KEYMASK_SHIFTLOCK );
+    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
+					    SPI_KEYMASK_SHIFTLOCK |
+					    SPI_KEYMASK_MOD2 );
+}
+
+/**
+ * ke_terminate
+ *
+ * Deregister key listener(s)
+ *
+**/
+void
+ke_terminate (void)
+{
+    sru_return_if_fail (ke_keyboard_status != KE_IDLE);
+    
+    sru_debug ("ke_terminate...");
+
+    /* unregister keyecho events */
+    ke_keyecho_unregister_events ();
+    
+    /* unregister layer events */
+    ke_layer_unregister_events ();
+    
+    /* unregister user_defined events */
+    ke_user_key_list_unregister_events ();  
+    
+    ke_user_key_list_free (ke_user_key_list);
     ke_user_key_list_free (reg_list);
+    
+    /* unref key listeners */
+    AccessibleKeystrokeListener_unref (ke_layer_listener);
+    AccessibleKeystrokeListener_unref (ke_keyecho_listener);
     AccessibleKeystrokeListener_unref (ke_user_key_listener);  
   
     SPI_freeAccessibleKeySet (ke_layer_keyset);
