@@ -217,7 +217,7 @@ srs_voice_set_rate (SRSVoice	*srs_voice,
 	srs_voice->rate += atoi(rate);
     }
 
-#ifdef SRSSPEECH_DEBUG
+#ifdef SRSSPEECH_PARAM_DEBUG
     fprintf (stderr, "\nsrspeech : Voice %s has rate = %d.",
 	    srs_voice->voice_id, 
 	    srs_voice->rate);
@@ -239,7 +239,7 @@ srs_voice_set_pitch (SRSVoice 	*srs_voice,
 	srs_voice->pitch += atoi(pitch);
     }
 	
-#ifdef SRSSPEECH_DEBUG
+#ifdef SRSSPEECH_PARAM_DEBUG
     fprintf (stderr, "\nsrspeech : Voice %s has pitch = %d.",
 	    srs_voice->voice_id, 
 	    srs_voice->pitch);
@@ -261,7 +261,7 @@ srs_voice_set_volume (SRSVoice *srs_voice,
 	srs_voice->volume += atoi(volume);
     }
 	
-#ifdef SRSSPEECH_DEBUG
+#ifdef SRSSPEECH_PARAM_DEBUG
     fprintf (stderr, "\nsrspeech : Voice %s has volume = %d.",
 	    srs_voice->voice_id, 
 	    srs_voice->volume);
@@ -530,10 +530,12 @@ tts_callback (TTSEvent 	event,
     	   break;
 	case TTS_EV_WORD_MARKER:
     	    /* noting to do yet */
-    	    fprintf (stderr, "Word marker\n");
+#ifdef SRSSPEECH_MARKER_DEBUG
+    	    sru_message ("Word marker\n");
+#endif
     	    break;
 	default:
-    	    fprintf (stderr, "Unknown TTS event\n");
+    	    sru_warning ("Unknown TTS event");
     	    break;
     }
 }
@@ -585,7 +587,7 @@ srs_speak_to_engine (SRSText *srs_text)
 		    current_engine->shut_up();
 		}
 	    }
-	    if (srv->cb_support)
+	    if (srv->cb_support == 1)
 		cb_support_temp = 1;
 	    else
 		cb_support_temp = 0;
@@ -622,7 +624,10 @@ srs_speak_optimization (void)
     static gboolean busy = FALSE;
     
     if (busy)
+    {
+	fprintf (stderr, "\nBUSY");
 	return FALSE;
+    }
     busy = TRUE;
 
     /* OPTIMIZATION FOR GS - concatenate texts with same 
