@@ -19,13 +19,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include "libsrconf.h"
 #include <gconf/gconf-client.h>
 #include <stdio.h>
 #include <string.h>
 #include "SRMessages.h"
+#include "srintl.h"
 
-#define NUM_OF_CONFIGURABLES (10)
 
 /**
  * srconf_fnc:
@@ -65,14 +66,6 @@ static SRConfStatusEnum srconf_status = SRCONF_IDLE;
 **/
 GConfClient *gconf_client=NULL;
 
-/**
- * cnx_id:
- *
- * Connection ID to GConf.
- *
-**/
-guint cnx_id[NUM_OF_CONFIGURABLES];
-
 /**
  * gconf_root_dir_path:
  *
@@ -88,26 +81,31 @@ static gchar *gconf_root_dir_path = NULL;
 **/
 #define DEFAULT_ROOT_DIR_PATH "/apps/gnopernicus"
 
+#define NUM_OF_CONFIGURABLES (10)
 /**
- * srconf_path
+ * srconf_notify_directories
  *
  * Configuration directories.
 **/
-static gchar *srconf_path[NUM_OF_CONFIGURABLES] = 
+struct _srconf_notify_directories
 {
-"/braille", 
-"/gnopi", 
-"/kbd_mouse", 
-"/magnifier", 
-"/speech/settings",
-"/srcore", 
-"/srlow", 
-"/command_map/keyboard/changes_end",
-"/presentation/changes_end",
-"/speech/voice"
+    gchar 		*notify_path;
+    SRConfigurablesEnum	 module;
+    guint		 cnx_id;
+} srconf_notify_directories[NUM_OF_CONFIGURABLES] = 
+{
+    {"/braille", 				CFGM_BRAILLE,		-1},
+    {"/gnopi", 					CFGM_GNOPI,		-1},
+    {"/kbd_mouse", 				CFGM_KBD_MOUSE,		-1},
+    {"/magnifier", 				CFGM_MAGNIFIER,		-1},
+    {"/srcore", 				CFGM_SRCORE,		-1},
+    {"/speech/settings",			CFGM_SPEECH,		-1},
+    {"/speech/voices_parameters",		CFGM_SPEECH_VOICE_PARAM,-1},
+    {"/speech/voice",				CFGM_SPEECH_VOICE,	-1},
+    {"/command_map/keyboard/changes_end", 	CFGM_KEY_PAD,		-1},
+    {"/presentation/changes_end",		CFGM_PRESENTATION,	-1}
 };
 
-
 /**
  * suspend_notify:
  *
@@ -136,226 +134,6 @@ srconf_free_slist (GSList *list)
     return TRUE;
 }
 
-/**
- * sr_config_structure_destructor:
- *
- * This function will be used to destroy data allocated in SREvent's data member.
-**/
-static void 
-sr_config_structure_destructor (gpointer data)
-{
-    SRConfigStructure *config = (SRConfigStructure *)data;
-
-    sru_return_if_fail (config);
-
-    g_free(config ->key);
-    g_free(config ->newvalue);
-    g_free(config);
-    config = NULL;
-}
-
-/**
- * sr_config_changed_callback
- *
- * callback for notification of configuration changes
- *
-**/
-static void 
-sr_config_changed_callback(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
-{
-    SREvent *evnt = NULL;
-    SRConfigStructure *SRstruct = NULL;
-
-    sru_return_if_fail (entry);
-    sru_return_if_fail (entry->value);
-    
-    SRstruct = (SRConfigStructure*) g_new0 (SRConfigStructure, 1);
-    sru_assert (SRstruct);
-    
-    sru_debug("srconf:Entry key:%s",gconf_entry_get_key(entry));
-    if (entry->value)
-    {
-	sru_debug("srconf:Entry value type:%i",entry->value->type);
-	switch(entry->value->type)
-	{	
-    	    case GCONF_VALUE_INT:    
-		sru_debug("srconf:Entry value:%i",gconf_value_get_int(entry->value));
-		break;
-    	    case GCONF_VALUE_FLOAT:  
-		sru_debug("srconf:Entry value:%f",gconf_value_get_float(entry->value));
-		break;
-    	    case GCONF_VALUE_STRING: 
-		sru_debug("srconf:Entry value:%s",gconf_value_get_string(entry->value));
-		break;
-    	    case GCONF_VALUE_BOOL:   
-		sru_debug ("srconf:Entry value:%i",gconf_value_get_bool(entry->value));
-		break;
-    	    case GCONF_VALUE_LIST: 
-	    	sru_debug ("srconf:list");
-		break;
-	    case GCONF_VALUE_INVALID:
-    	    default:
-	    	sru_debug ("srconf:unset");
-		break;
-	}
-    }
-
-    evnt = sre_new ();
-    sru_assert (evnt);
-    
-    evnt->type = SR_EVENT_CONFIG_CHANGED;
-    SRstruct->module = (int)user_data;
-    SRstruct->key = g_strdup (gconf_entry_get_key(entry) +
-			      strlen(gconf_root_dir_path) +
-			      strlen(srconf_path[(guint)user_data - 1]) + 1);
-    if (entry->value)
-    {
-	switch(entry->value->type)
-	{
-    	    case GCONF_VALUE_INT:
-		SRstruct->type = CFGT_INT;
-		SRstruct->newvalue = (gpointer)g_new0 (gint, 1);
-		*((gint*)SRstruct->newvalue) = gconf_value_get_int(entry->value);
-		break;
-    	    case GCONF_VALUE_FLOAT:
-		SRstruct->type = CFGT_FLOAT;
-		SRstruct->newvalue = (gpointer)g_new0 (gfloat, 1);
-		*((gfloat*)SRstruct->newvalue) = gconf_value_get_float(entry->value);
-		break;
-    	    case GCONF_VALUE_STRING:
-		SRstruct->type = CFGT_STRING;
-		SRstruct->newvalue = (gpointer)g_strdup(gconf_value_get_string(entry->value));
-		break;
-    	    case GCONF_VALUE_BOOL:
-		SRstruct->type = CFGT_BOOL;
-		SRstruct->newvalue = (gpointer)g_new0 (gboolean, 1);
-		*((gboolean*)SRstruct->newvalue) = gconf_value_get_bool(entry->value);
-		break;
-	    case GCONF_VALUE_LIST: 
-		SRstruct->type = CFGT_LIST;
-		SRstruct->newvalue = NULL; /* NOW IT IS'T RETURN ANY LIST */
-		break;
-	    case GCONF_VALUE_INVALID:
-		break;
-	    default:
-		break;
-	}
-    }
-    else
-    {
-	SRstruct->type = CFGT_UNSET;
-	SRstruct->newvalue = NULL;
-    }
-		
-    evnt->data = SRstruct;
-    evnt->data_destructor = sr_config_structure_destructor;
-
-    if (srconf_fnc) 
-        srconf_fnc (evnt,0);
-    
-    sre_release_reference (evnt);
-    sru_debug ("srconf:SendEventSrCore");
-}
-/**
- * srconf_init:
- *
- * This function initialize Gconf and register the notifications
- *	srconfcb - the callback function used to send data to caller
- *	gconf_root_dir_path - the root directory of GConf
-**/
-gboolean 
-srconf_init (SRConfCB srconfcb, const gchar *_gconf_root_dir_path)
-{
-    gint i;
-    GError *error = NULL;    
-    gchar *path = NULL;
-    
-    sru_return_val_if_fail (srconf_status == SRCONF_IDLE, FALSE);
-    sru_return_val_if_fail (srconfcb != NULL, FALSE);
-
-    srconf_fnc = srconfcb;
-
-    gconf_root_dir_path = g_strdup (_gconf_root_dir_path?_gconf_root_dir_path: DEFAULT_ROOT_DIR_PATH);
-    
-    sru_return_val_if_fail (gconf_root_dir_path != NULL, FALSE);
-
-    /*set a default gconf client		      */
-    gconf_client = gconf_client_get_default ();
-    
-    /*adding the directory to be watched		      */
-    gconf_client_add_dir (gconf_client, gconf_root_dir_path, GCONF_CLIENT_PRELOAD_RECURSIVE, &error);
-    
-    
-    if (error != NULL)
-    {
-	sru_warning("srconf:Failed to add dir.\n");
-	sru_warning("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
-	g_error_free(error);
-	error = NULL;
-    }
-	
-    /*adding notification callbacks*/
-    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
-    {
-    	path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_path[i]);
-    	cnx_id[i] = gconf_client_notify_add (gconf_client, path, sr_config_changed_callback, (gpointer)(i+1), NULL, &error);
-	
-	if (error != NULL)
-	{
-	    sru_warning ("srconf:Failed to add notify.\n");
-	    sru_warning ("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
-	    g_error_free (error);
-	    error = NULL;
-	}
-
-    	g_free (path);
-    	path = NULL;
-    }
-        
-    srconf_status = SRCONF_RUNNING;
-    sru_debug ("SRConf running.");
-            
-    return TRUE;
-}
-
-/**
- * srconf_terminate:
- *
- * Deregister notifications and destroy additional objects
- *
-**/
-void 
-srconf_terminate()
-{
-    int i;
-    GError *error = NULL;
-    sru_return_if_fail (srconf_status == SRCONF_RUNNING);
-    
-    /* removing notification callbacks */
-    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
-    	gconf_client_notify_remove (gconf_client, cnx_id[i]);
-    	
-    /* remove the watching directory */
-    gconf_client_remove_dir (gconf_client, gconf_root_dir_path, &error);
-    
-    if (error != NULL)
-    {
-	sru_warning("srconf:Failed to remove dir.\n");
-	sru_warning("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
-	g_error_free(error);
-	error = NULL;
-    }
-
-    g_free (gconf_root_dir_path);
-    gconf_root_dir_path = NULL;
-
-    srconf_fnc = NULL;
-
-    srconf_status = SRCONF_IDLE;
-    sru_debug ("SRConf idle.");
-}
-
-
 /**
  *
  * Convert SRConfigTypesEnum to GConfValueType
@@ -418,6 +196,271 @@ srconf_convert_GConfValueType_to_SRConfigTypesEnum(GConfValueType type)
     }
 }
 
+
+/**
+ * sr_config_structure_destructor:
+ *
+ * This function will be used to destroy data allocated in SREvent's data member.
+**/
+static void 
+sr_config_structure_destructor (gpointer data)
+{
+    SRConfigStructure *config = (SRConfigStructure *)data;
+
+    sru_return_if_fail (config);
+
+    g_free (config->key);
+    if (config->type == CFGT_LIST)
+    {
+	GSList *tmp = NULL;
+	for (tmp = (GSList*)config->newvalue; tmp ; tmp = tmp->next)
+	{
+	    if (tmp->data)
+		g_free (tmp->data);
+	}
+	g_slist_free (config->newvalue);
+	config->newvalue = NULL;
+    }
+    else
+	g_free (config->newvalue);
+    g_free (config);
+    config = NULL;
+}
+
+/**
+ * sr_config_changed_callback
+ *
+ * callback for notification of configuration changes
+ *
+**/
+static void 
+sr_config_changed_callback (GConfClient *client, 
+			    guint cnxn_id, 
+			    GConfEntry *entry, 
+			    gpointer user_data)
+{
+    SREvent *evnt = NULL;
+    SRConfigStructure *SRstruct = NULL;
+
+    sru_return_if_fail (entry);
+    sru_return_if_fail (entry->value);
+    
+    SRstruct = (SRConfigStructure*) g_new0 (SRConfigStructure, 1);
+    sru_assert (SRstruct);
+
+    sru_debug("Entry key:%s", gconf_entry_get_key (entry));
+    if (entry->value)
+    {
+	sru_debug("Entry value type:%i",entry->value->type);
+	switch(entry->value->type)
+	{	
+    	    case GCONF_VALUE_INT:    
+		sru_debug("Entry value:%i",gconf_value_get_int(entry->value));
+		break;
+    	    case GCONF_VALUE_FLOAT:  
+		sru_debug("Entry value:%f",gconf_value_get_float(entry->value));
+		break;
+    	    case GCONF_VALUE_STRING: 
+		sru_debug ("Entry value:%s",gconf_value_get_string(entry->value));
+		break;
+    	    case GCONF_VALUE_BOOL:   
+		sru_debug ("Entry value:%i",gconf_value_get_bool(entry->value));
+		break;
+    	    case GCONF_VALUE_LIST: 
+	    	sru_debug ("list");
+		break;
+	    case GCONF_VALUE_INVALID:
+    	    default:
+	    	sru_debug ("unset");
+		break;
+	}
+    }
+
+    evnt = sre_new ();
+    sru_assert (evnt);
+    
+    evnt->type = SR_EVENT_CONFIG_CHANGED;
+    SRstruct->module = srconf_notify_directories[(gint)user_data].module;
+    if (strlen (gconf_entry_get_key (entry)) >
+	strlen (gconf_root_dir_path) +
+	strlen (srconf_notify_directories[(guint)user_data].notify_path))
+    {    
+	SRstruct->key = g_strdup (gconf_entry_get_key (entry) +
+			    	  strlen (gconf_root_dir_path) +
+			          strlen (srconf_notify_directories[(guint)user_data].notify_path) + 1);
+    }
+    else
+	SRstruct->key = g_path_get_basename (gconf_entry_get_key(entry));
+	
+    if (entry->value)
+    {
+	switch(entry->value->type)
+	{
+    	    case GCONF_VALUE_INT:
+		SRstruct->type = CFGT_INT;
+		SRstruct->newvalue = (gpointer)g_new0 (gint, 1);
+		*((gint*)SRstruct->newvalue) = gconf_value_get_int (entry->value);
+		break;
+    	    case GCONF_VALUE_FLOAT:
+		SRstruct->type = CFGT_FLOAT;
+		SRstruct->newvalue = (gpointer)g_new0 (gfloat, 1);
+		*((gfloat*)SRstruct->newvalue) = gconf_value_get_float (entry->value);
+		break;
+    	    case GCONF_VALUE_STRING:
+		SRstruct->type = CFGT_STRING;
+		SRstruct->newvalue = (gpointer)g_strdup (gconf_value_get_string (entry->value));
+		break;
+    	    case GCONF_VALUE_BOOL:
+		SRstruct->type = CFGT_BOOL;
+		SRstruct->newvalue = (gpointer)g_new0 (gboolean, 1);
+		*((gboolean*)SRstruct->newvalue) = gconf_value_get_bool (entry->value);
+		break;
+	    case GCONF_VALUE_LIST: 
+	    {
+		GSList	*iter     = NULL;
+		GSList  *new_list = NULL;
+		SRConfigTypesEnum type;
+		
+		type = srconf_convert_GConfValueType_to_SRConfigTypesEnum (gconf_value_get_list_type (entry->value));
+		
+		if (type != CFGT_STRING)
+		    break;
+		    		
+		for(iter = gconf_value_get_list (entry->value) ; iter ; iter = iter->next)
+		{
+		    GConfValue *v_data = iter->data;
+		    				
+		    new_list = g_slist_append (new_list,
+						g_strdup((gchar*)gconf_value_get_string(v_data)));
+		}
+		
+		SRstruct->type = CFGT_LIST;
+		SRstruct->newvalue = (gpointer)new_list;
+		
+		break;
+	    }
+	    case GCONF_VALUE_INVALID:
+		break;
+	    default:
+		break;
+	}
+    }
+    else
+    {
+	SRstruct->type = CFGT_UNSET;
+	SRstruct->newvalue = NULL;
+    }
+		
+    evnt->data = SRstruct;
+    evnt->data_destructor = sr_config_structure_destructor;
+
+    if (srconf_fnc) 
+        srconf_fnc (evnt,0);
+
+    sre_release_reference (evnt);
+    sru_debug ("SendEventSrCore");
+}
+/**
+ * srconf_init:
+ *
+ * This function initialize Gconf and register the notifications
+ *	srconfcb - the callback function used to send data to caller
+ *	gconf_root_dir_path - the root directory of GConf
+**/
+gboolean 
+srconf_init (SRConfCB srconfcb, const gchar *_gconf_root_dir_path)
+{
+    gint i;
+    GError *error = NULL;    
+    gchar *path = NULL;
+    
+    sru_return_val_if_fail (srconf_status == SRCONF_IDLE, FALSE);
+    sru_return_val_if_fail (srconfcb != NULL, FALSE);
+
+    srconf_fnc = srconfcb;
+
+    gconf_root_dir_path = g_strdup (_gconf_root_dir_path?_gconf_root_dir_path: DEFAULT_ROOT_DIR_PATH);
+    
+    sru_return_val_if_fail (gconf_root_dir_path != NULL, FALSE);
+
+    /*set a default gconf client		      */
+    gconf_client = gconf_client_get_default ();
+    
+    /*adding the directory to be watched		      */
+    gconf_client_add_dir (gconf_client, gconf_root_dir_path, GCONF_CLIENT_PRELOAD_ONELEVEL, &error);
+    
+    
+    if (error != NULL)
+    {
+	sru_warning(_("Failed to add directory."));
+	sru_warning(_(error->message));
+	g_error_free(error);
+	error = NULL;
+    }
+	
+    /*adding notification callbacks*/
+    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+    {
+    	path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_notify_directories[i].notify_path);
+    	srconf_notify_directories[i].cnx_id = gconf_client_notify_add (gconf_client, path, sr_config_changed_callback, (gpointer)(i), NULL, &error);
+	
+	if (error != NULL)
+	{
+	    sru_warning (_("Failed to add notify."));
+	    sru_warning (_(error->message));
+	    g_error_free (error);
+	    error = NULL;
+	}
+
+    	g_free (path);
+    	path = NULL;
+    }
+        
+    srconf_status = SRCONF_RUNNING;
+    sru_debug ("SRConf running.");
+            
+    return TRUE;
+}
+
+/**
+ * srconf_terminate:
+ *
+ * Deregister notifications and destroy additional objects
+ *
+**/
+void 
+srconf_terminate()
+{
+    int i;
+    GError *error = NULL;
+    sru_return_if_fail (srconf_status == SRCONF_RUNNING);
+    
+    /* removing notification callbacks */
+    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+    	gconf_client_notify_remove (gconf_client, srconf_notify_directories[i].cnx_id);
+    	
+    /* remove the watching directory */
+    gconf_client_remove_dir (gconf_client, gconf_root_dir_path, &error);
+    
+    if (error != NULL)
+    {
+	sru_warning (_("Failed to remove directory."));
+	sru_warning (_(error->message));
+	g_error_free (error);
+	error = NULL;
+    }
+
+    g_free (gconf_root_dir_path);
+    gconf_root_dir_path = NULL;
+
+    srconf_fnc = NULL;
+
+    srconf_status = SRCONF_IDLE;
+    sru_debug ("SRConf idle.");
+}
+
+
+
 /**
  * srconf_get_config_data_with_default
  *
@@ -437,7 +480,6 @@ srconf_get_config_data_with_default(const gchar *key, SRConfigTypesEnum conftype
 {
     GError 	*error = NULL;
     gchar 	*path = NULL;
-    GSList	*list_value = NULL;
     GConfValue	*value = NULL;
     gboolean	used_default;
     
@@ -446,10 +488,10 @@ srconf_get_config_data_with_default(const gchar *key, SRConfigTypesEnum conftype
     sru_return_val_if_fail (srconf_status == SRCONF_RUNNING, FALSE );
     sru_return_val_if_fail ((confmodule >= 1) && (confmodule < NUM_OF_CONFIGURABLES), FALSE);
     
-    path = g_strdup_printf("%s%s/%s",gconf_root_dir_path,srconf_path[confmodule-1],key);
+    path = g_strdup_printf("%s%s/%s",gconf_root_dir_path,srconf_notify_directories[confmodule-1].notify_path,key);
     sru_return_val_if_fail (path != NULL, FALSE);
     sru_debug ("srconf_get_config_data_with_default:Path:%s", path);        
-    value = gconf_client_get(gconf_client,path,&error);
+    value = gconf_client_get (gconf_client, path, &error);
     
     g_free (path);
 
@@ -484,12 +526,10 @@ srconf_get_config_data_with_default(const gchar *key, SRConfigTypesEnum conftype
 			    GSList *iter = NULL;
 			    GSList *list = NULL;
 			    SRConfigTypesEnum type;
-			    
-			    list_value = gconf_value_get_list (value);
-			    
+			    			    
 			    type = srconf_convert_GConfValueType_to_SRConfigTypesEnum (gconf_value_get_list_type (value));
 			    
-			    for(iter = list_value ; iter ; iter = iter->next)
+			    for (iter = gconf_value_get_list (value) ; iter ; iter = iter->next)
 			    {
 				GConfValue *v_data = iter->data;
 				
@@ -568,7 +608,6 @@ srconf_get_data_with_default(const gchar *key, SRConfigTypesEnum conftype, gpoin
 {
     GError 	*error = NULL;
     GConfValue	*value = NULL;
-    GSList	*list_value = NULL;
     gchar 	*path = NULL;
     gboolean	used_default;
         
@@ -616,10 +655,9 @@ srconf_get_data_with_default(const gchar *key, SRConfigTypesEnum conftype, gpoin
 			    GSList *list = NULL;
 			    SRConfigTypesEnum type;
 			    
-			    list_value = gconf_value_get_list (value);			    
 			    type = srconf_convert_GConfValueType_to_SRConfigTypesEnum (gconf_value_get_list_type (value));
 			    
-			    for(iter = list_value ; iter ; iter = iter->next)
+			    for(iter = gconf_value_get_list (value) ; iter ; iter = iter->next)
 			    {
 				GConfValue *v_data = iter->data;
 								
@@ -701,7 +739,7 @@ srconf_set_config_data(const gchar *key, SRConfigTypesEnum conftype, const gpoin
     sru_return_val_if_fail ((confmodule >= 1) && (confmodule < NUM_OF_CONFIGURABLES), FALSE);    
     sru_return_val_if_fail (key != NULL, FALSE);
     
-    path = g_strdup_printf("%s%s/%s",gconf_root_dir_path,srconf_path[confmodule-1],key);
+    path = g_strdup_printf("%s%s/%s",gconf_root_dir_path,srconf_notify_directories[confmodule-1].notify_path,key);
     sru_return_val_if_fail (path != NULL, FALSE );
     sru_return_val_if_fail (gconf_client_key_is_writable (gconf_client,path,NULL), FALSE);
     switch(conftype)
@@ -746,8 +784,8 @@ srconf_set_config_data(const gchar *key, SRConfigTypesEnum conftype, const gpoin
     
     if (error != NULL)
     {
-        sru_warning ("srconf:Failed to set configdata");
-        sru_warning ("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
+        sru_warning (_("Failed to set configdata."));
+        sru_warning (_(error->message));
         g_error_free (error);
         error = NULL;
         ret = FALSE;
@@ -820,8 +858,8 @@ srconf_set_data(const gchar *key, SRConfigTypesEnum conftype, const gpointer dat
     
     if (error != NULL)
     {
-        sru_warning ("srconf:Failed set data");
-        sru_warning ("Recomanded to delete ~/.gconf/apps/gnopernicus and ~/.gconfd/lock directories");
+        sru_warning (_("Failed set data."));
+        sru_warning (_(error->message));
         g_error_free(error);
         error = NULL;
         ret = FALSE;
@@ -842,7 +880,7 @@ srconf_unset_key (const gchar *key,const gchar *section)
     GError *error = NULL;
     GConfValue *val = NULL;
     gchar *path = NULL;
-    gboolean retval;
+    gboolean retval = TRUE;
     
     sru_return_val_if_fail (section != NULL, FALSE);
     sru_return_val_if_fail (key != NULL, FALSE);
@@ -859,7 +897,7 @@ srconf_unset_key (const gchar *key,const gchar *section)
         
 	if (error != NULL)
 	{
-	    sru_warning ("srconf:Failed unset key");
+	    sru_warning ("Failed unset key.");
 	    g_error_free(error);
 	    error = NULL;
 	}
