@@ -36,11 +36,16 @@
 #include "screen-review.h"
 #include "srintl.h"
 
+#undef SRCTRL_DEBUG
+
 extern gboolean src_use_magnifier;
 extern gboolean src_use_speech;
 extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
 extern gboolean src_enable_format_string;
+extern gchar   *src_braille_translation_table;
+extern gint	src_braille_position_sensor;
+extern gint	src_braille_optical_sensor;
 
 gboolean  src_button_left_pressed   = FALSE;
 gboolean  src_button_middle_pressed = FALSE;
@@ -383,16 +388,17 @@ src_mag_cursor_toggle ()
 }
 
 static gboolean
-src_mag_cursor_decrease_size ()
+src_mag_cursor_adjust_size (gint offset)
 {
-	gboolean cursor;
-	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+    gboolean cursor;
+    gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
+    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+				       CFGT_BOOL, 
+				       &cursor, 
+				       (gpointer)&default_cursor,
+				       MAGNIFIER_CONFIG_PATH)
+	) return FALSE;
+
     if (src_use_magnifier && cursor) 
     {
 	gint default_val = DEFAULT_MAGNIFIER_CURSOR_SIZE;
@@ -405,8 +411,10 @@ src_mag_cursor_decrease_size ()
 					   MAGNIFIER_CONFIG_PATH)
 	    ) return FALSE;
 
-	cursor_size = ((cursor_size - 4) < MIN_CURSOR_SIZE) ? MIN_CURSOR_SIZE : cursor_size - 4;
-	
+	cursor_size = CLAMP (cursor_size + offset,
+			     MIN_CROSSWIRE_SIZE,
+			     MAX_CROSSWIRE_SIZE);
+
 	if (!srconf_set_data (MAGNIFIER_CURSOR_SIZE, 
 			      CFGT_INT, 
 			      &cursor_size, 
@@ -416,40 +424,16 @@ src_mag_cursor_decrease_size ()
     return TRUE;
 }
 
+static gboolean
+src_mag_cursor_decrease_size ()
+{
+    return src_mag_cursor_adjust_size (-4);
+}
+
 static gboolean
 src_mag_cursor_increase_size ()
 {
-	gboolean cursor;
-	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-    if (src_use_magnifier && cursor) 
-    {
-	gint default_val = DEFAULT_MAGNIFIER_CURSOR_SIZE;
-	gint cursor_size;
-	
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR_SIZE, 
-					   CFGT_INT, 
-					   &cursor_size, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-	
-	cursor_size = ((cursor_size + 4) > MAX_CURSOR_SIZE) ? MAX_CURSOR_SIZE : cursor_size + 4;
-	
-	if (!srconf_set_data (MAGNIFIER_CURSOR_SIZE,
-			      CFGT_INT, 
-			      &cursor_size,
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-    }
-    return TRUE;
+    return src_mag_cursor_adjust_size (4);
 }
 
 static gboolean
@@ -487,23 +471,23 @@ src_mag_cursor_mag_on_off ()
 }
 
 static gboolean
-src_mag_crosswire_decrease_size ()
+src_mag_crosswire_adjust_size (guint offset)
 {
-	gboolean cursor, crosswire;
-	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-	gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
-					   CFGT_BOOL, 
-					   &crosswire, 
-					   (gpointer)&default_crosswire,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
+    gboolean cursor, crosswire;
+    gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
+    gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;
+    if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
+				       CFGT_BOOL, 
+				       &cursor, 
+				       (gpointer)&default_cursor,
+				       MAGNIFIER_CONFIG_PATH)
+	) return FALSE;
+    if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
+				       CFGT_BOOL, 
+				       &crosswire, 
+				       (gpointer)&default_crosswire,
+				       MAGNIFIER_CONFIG_PATH)
+	) return FALSE;
 
     if (src_use_magnifier && cursor && crosswire)
     {
@@ -517,7 +501,9 @@ src_mag_crosswire_decrease_size ()
 					   MAGNIFIER_CONFIG_PATH)
 	    ) return FALSE;
 
-	crosswire_size = ((--crosswire_size ) < MIN_CROSSWIRE_SIZE) ? MIN_CROSSWIRE_SIZE : crosswire_size;
+	crosswire_size = CLAMP (crosswire_size + offset,
+				MIN_CROSSWIRE_SIZE,
+				MAX_CROSSWIRE_SIZE);
 
 	if (!srconf_set_data (MAGNIFIER_CROSSWIRE_SIZE, 
 			      CFGT_INT, 
@@ -528,47 +514,16 @@ src_mag_crosswire_decrease_size ()
     return TRUE;
 }
 
+static gboolean
+src_mag_crosswire_decrease_size ()
+{
+	return src_mag_crosswire_adjust_size (-1);
+}
+
 static gboolean
 src_mag_crosswire_increase_size ()
 {
-	gboolean cursor, crosswire;
-	gboolean default_cursor = DEFAULT_MAGNIFIER_CURSOR;
-	gboolean default_crosswire = DEFAULT_MAGNIFIER_CROSSWIRE;
-	if (!srconf_get_data_with_default (MAGNIFIER_CURSOR, 
-					   CFGT_BOOL, 
-					   &cursor, 
-					   (gpointer)&default_cursor,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE, 
-					   CFGT_BOOL, 
-					   &crosswire, 
-					   (gpointer)&default_crosswire,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-    if (src_use_magnifier && cursor && crosswire)
-    {
-	gint default_val = DEFAULT_MAGNIFIER_CROSSWIRE_SIZE;
-	gint crosswire_size;
-
-	if (!srconf_get_data_with_default (MAGNIFIER_CROSSWIRE_SIZE, 
-					   CFGT_INT, 
-					   &crosswire_size, 
-					   (gpointer)&default_val,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-
-
-	crosswire_size = ((++crosswire_size) > MAX_CROSSWIRE_SIZE) ? MAX_CROSSWIRE_SIZE : crosswire_size;
-
-	if (!srconf_set_data (MAGNIFIER_CROSSWIRE_SIZE,
-			      CFGT_INT, 
-			      &crosswire_size,
-			      MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-    }
-    return TRUE;
+	return src_mag_crosswire_adjust_size (+1);
 }
 
 static gboolean
@@ -748,10 +703,6 @@ src_mag_default_all ()
 {
     if (src_use_magnifier) 
     {
-/*	fprintf (stderr, "%s %d Get defaults",
-			__FILE__,
-			__LINE__);
-*/
 	src_magnifier_config_get_defaults ();
     }
     return TRUE;
@@ -771,17 +722,6 @@ src_mag_smoothing_toggle ()
     
     if (src_use_magnifier) 
     {
-/*	gchar *_smoothing;
-	gchar *default_smoothing = DEFAULT_MAGNIFIER_SMOOTHING;
-	if (!srconf_get_data_with_default (MAGNIFIER_SMOOTHING, 
-    					   CFGT_STRING, 
-					   _smoothing, 
-					   (gpointer)default_smoothing,
-					   MAGNIFIER_CONFIG_PATH)
-	    ) return FALSE;
-	fprintf(stderr, "\nSR: magnifier smoothing type from gconf %s\n",
-			_smoothing);
-*/
 	if (!srconf_set_data (MAGNIFIER_SMOOTHING,
 			      CFGT_STRING, 
 			      smoothing[index],
@@ -904,19 +844,19 @@ extern gboolean src_mouse_click;
 static gint src_nav_mode = SR_NAV_MODE_WINDOW;
 
 static gboolean
-src_nav_parent_logic ()
+src_nav_logic (SRNavigationDir dir, const char *missing_node)
 {
-    SRObject *parent;
+    SRObject *node;
     gboolean rv = FALSE;
     
     if (!src_crt_sro)
 	return FALSE;
     
-    sro_get_sro (src_crt_sro, SR_NAV_PARENT, &parent, src_nav_mode);
-    if (parent)
+    sro_get_sro (src_crt_sro, dir, &node, src_nav_mode);
+    if (node)
     {
 	sro_release_reference (src_crt_sro);
-	src_crt_sro = parent;
+	src_crt_sro = node;
 	src_cmd_queue_add ("present current object");
 	if (src_mouse_take)
 	{
@@ -930,7 +870,7 @@ src_nav_parent_logic ()
     else
     {
 	if (src_use_speech)
-	    src_say_message (_("no parent")); 
+            src_say_message (missing_node);
     }
 
     return rv;
@@ -962,7 +902,7 @@ src_nav_parent ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_parent_flat ();
     else
-	rv = src_nav_parent_logic ();
+        rv = src_nav_logic (SR_NAV_PARENT, _("no parent"));
 
     return rv;
 }
@@ -978,42 +918,6 @@ src_nav_child_flat ()
     return rv;
 }
 
-
-static gboolean
-src_nav_child_logic ()
-{
-    SRObject *child;
-    gboolean rv = FALSE;
-    
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_CHILD, &child, src_nav_mode);
-    if (child)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = child;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no children")); 
-    }
-
-    return rv;
-}
-
-
-
 static gboolean
 src_nav_child ()
 {
@@ -1022,12 +926,11 @@ src_nav_child ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_child_flat ();
     else
-	rv = src_nav_child_logic ();
+	rv = src_nav_logic (SR_NAV_CHILD, _("no children"));
 
     return rv;
 }
 
-
 static gboolean src_flat_review_line (gint index);
 
 static gboolean
@@ -1044,39 +947,6 @@ src_nav_next_flat ()
     return rv;
 }
 
-
-static gboolean
-src_nav_next_logic ()
-{
-    SRObject *next;
-    gboolean rv = FALSE;
-
-    if (!src_crt_sro)
-	return FALSE;
-    sro_get_sro (src_crt_sro, SR_NAV_NEXT, &next, src_nav_mode);
-    if (next)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = next;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no next")); 
-    }
-
-    return rv;
-}
-
 static gboolean
 src_nav_next ()
 {
@@ -1085,7 +955,7 @@ src_nav_next ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_next_flat ();
     else
-	rv = src_nav_next_logic ();
+	rv = src_nav_logic (SR_NAV_NEXT, _("no next"));
 
     return rv;
 }
@@ -1271,8 +1141,6 @@ src_find_next ()
 }
 
 
-
-/* csongor */
 static gboolean
 src_find_set ()
 {
@@ -1300,41 +1168,6 @@ src_nav_previous_flat ()
     return rv;
 }    
 
-
-
-static gboolean
-src_nav_previous_logic ()
-{
-    SRObject *previous;
-    gboolean rv = FALSE;
-    
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_PREV, &previous, src_nav_mode);
-    if (previous)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = previous;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no previous")); 
-    }
-
-    return rv;
-}
-
 static gboolean
 src_nav_previous ()
 {
@@ -1343,7 +1176,7 @@ src_nav_previous ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_previous_flat ();
     else
-	rv = src_nav_previous_logic ();
+	rv = src_nav_logic (SR_NAV_PREV, _("no previous"));
 
     return rv;
 }
@@ -1356,42 +1189,7 @@ src_nav_caret_flat ()
     return rv;
 }
 
-
-static gboolean
-src_nav_caret_logic ()
-{
-    SRObject *caret;
-    gboolean rv = FALSE;
-    
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_CARET, &caret, src_nav_mode);
-    if (caret)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = caret;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no caret")); 
-    }
-
-    return rv;
-}
-
-
-extern SRLong src_text_index;
+SRLong src_text_index;
 static gboolean
 src_caret_attributes ()
 {
@@ -1420,12 +1218,11 @@ src_nav_caret ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_caret_flat ();
     else
-	rv = src_nav_caret_logic ();
+	rv = src_nav_logic (SR_NAV_CARET, _("no caret"));
 
     return rv;
 }
 
-
 static gboolean
 src_nav_first_flat ()
 {
@@ -1434,40 +1231,6 @@ src_nav_first_flat ()
     return rv;
 }
 
-
-static gboolean
-src_nav_first_logic ()
-{
-    SRObject *first;
-    gboolean rv = FALSE;
-    
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_FIRST, &first, src_nav_mode);
-    if (first)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = first;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("already on first")); 
-    }
-
-    return rv;
-}
-
 static gboolean
 src_nav_first ()
 {
@@ -1476,12 +1239,11 @@ src_nav_first ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_first_flat ();
     else
-	rv = src_nav_first_logic ();
+	rv = src_nav_logic (SR_NAV_FIRST, _("already on first"));
 
     return rv;
 }
 
-
 static gboolean
 src_nav_last_flat ()
 {
@@ -1490,41 +1252,6 @@ src_nav_last_flat ()
     return rv;
 }
 
-
-static gboolean
-src_nav_last_logic ()
-{
-    SRObject *last;
-    gboolean rv = FALSE;
-    
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_LAST, &last, src_nav_mode);
-    if (last)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = last;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("already on last")); 
-    }
-
-    return rv;
-}
-
-
 static gboolean
 src_nav_last ()
 {
@@ -1533,12 +1260,11 @@ src_nav_last ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_last_flat ();
     else
-	rv = src_nav_last_logic ();
+	rv = src_nav_logic (SR_NAV_LAST, _("already on last"));
 
     return rv;
 }
 
-
 static gboolean
 src_nav_change_mode ()
 {
@@ -1547,20 +1273,20 @@ src_nav_change_mode ()
     {
 	case SR_NAV_MODE_WINDOW:
 	    src_nav_mode = SR_NAV_MODE_APPLICATION;
-	    message = _("navigation mode application");
+	    message = _("now navigating in application");
 	    break;
 	case SR_NAV_MODE_APPLICATION:
 	    src_nav_mode = SR_NAV_MODE_DESKTOP;
-	    message = _("navigation mode desktop");
+	    message = _("now navigating in desktop");
 	    break;
 	default:
 	    src_nav_mode = SR_NAV_MODE_WINDOW;
-	    message = _("navigation mode window");
+	    message = _("now navigating in window");
 	    break;    
     }
     if (src_use_speech && message)
     {
-	    src_say_message (message); 
+	src_say_message (message); 
     }
 
     return TRUE;
@@ -1572,6 +1298,11 @@ static gchar *src_last_cmd = NULL;
 extern gint src_speech_mode;
 static gboolean src_repeat_cnt = 1;
 
+static gint flat_review_x = 0, 
+	    flat_review_y = 0;
+static gint n_lines = 0;
+
+
 static gboolean
 src_repeat_last ()
 {
@@ -1662,39 +1393,9 @@ static gboolean
 src_nav_title_flat ()
 {
     gboolean rv = FALSE;
-
-    return rv;
-}
-
-static gboolean
-src_nav_title_logic ()
-{
-    SRObject *title;
-    gboolean rv = FALSE;
     
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_TITLE, &title, src_nav_mode);
-    if (title)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = title;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no title")); 
-    }
+    crt_line = 0;	
+    rv = src_flat_review_next_line();
 
     return rv;
 }
@@ -1707,7 +1408,7 @@ src_nav_title ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_title_flat ();
     else
-	rv = src_nav_title_logic ();
+	rv = src_nav_logic (SR_NAV_TITLE, _("no title"));
 
     return rv;
 }
@@ -1720,38 +1421,6 @@ src_nav_menubar_flat ()
     return rv;
 }
 
-static gboolean
-src_nav_menubar_logic ()
-{
-    SRObject *menu;
-    gboolean rv = FALSE;
-    
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_MENU, &menu, src_nav_mode);
-    if (menu)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = menu;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no menu bar")); 
-    }
-    return rv;
-}
-
 static gboolean
 src_nav_menubar ()
 {
@@ -1760,7 +1429,7 @@ src_nav_menubar ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_menubar_flat ();
     else
-	rv = src_nav_menubar_logic ();
+	rv = src_nav_logic (SR_NAV_MENU, _("no menu bar"));
 
     return rv;
 }
@@ -1769,44 +1438,12 @@ static gboolean
 src_nav_toolbar_flat ()
 {
     gboolean rv = FALSE;
+    crt_line = n_lines;
+    rv = src_flat_review_next_line();
 
     return rv;
 }
 
-static gboolean
-src_nav_toolbar_logic ()
-{
-    SRObject *toolbar;
-    gboolean rv = FALSE;
-    
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_TOOLBAR, &toolbar, src_nav_mode);
-    if (toolbar)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = toolbar;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no tool bar")); 
-    }
-
-    return rv;
-}
-
-
 static gboolean
 src_nav_toolbar ()
 {
@@ -1815,7 +1452,7 @@ src_nav_toolbar ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_toolbar_flat ();
     else
-	rv = src_nav_toolbar_logic ();
+	rv = src_nav_logic (SR_NAV_TOOLBAR, _("no tool bar"));
 
     return rv;
 }
@@ -1824,39 +1461,9 @@ static gboolean
 src_nav_statusbar_flat ()
 {
     gboolean rv = FALSE;
-
-    return rv;
-}
-
-static gboolean
-src_nav_statusbar_logic ()
-{
-    SRObject *statusbar;
-    gboolean rv = FALSE;
     
-    if (!src_crt_sro)
-	return FALSE;
-    
-    sro_get_sro (src_crt_sro, SR_NAV_STATUSBAR, &statusbar, src_nav_mode);
-    if (statusbar)
-    {
-	sro_release_reference (src_crt_sro);
-	src_crt_sro = statusbar;
-	src_cmd_queue_add ("present current object");
-	if (src_mouse_take)
-	{
-	    src_cmd_queue_add ("mouse goto current");
-	    if (src_mouse_click)
-		src_cmd_queue_add ("mouse left click");
-	}
-	src_cmd_queue_process ();
-	rv = TRUE;
-    }
-    else
-    {
-	if (src_use_speech)
-	    src_say_message (_("no status bar")); 
-    }
+    crt_line = screen_review_get_focused_line ();	
+    rv = src_flat_review_next_line();
 
     return rv;
 }
@@ -1869,7 +1476,7 @@ src_nav_statusbar ()
     if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
 	rv = src_nav_statusbar_flat ();
     else
-	rv = src_nav_statusbar_logic ();
+	rv = src_nav_logic (SR_NAV_STATUSBAR, _("no status bar"));
 
     return rv;
 }
@@ -2469,7 +2076,7 @@ src_surroundings_logic ()
 	sprintf (cnt, "%d", GPOINTER_TO_INT (crt2->data));
 	tmp2 = src_xml_make_part ("TEXT", src_speech_get_voice ("childcount"), cnt);
 	tmp3 = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar *)src_pres_get_role_name_for_speech ((gchar*) crt->data));
-	tmp4 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), GPOINTER_TO_INT (crt2->data) == 1 ? _("item") : _("items"));
+	tmp4 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), ngettext ("item", "items", GPOINTER_TO_INT (crt2->data)));
 	message6 = g_strconcat (message6, tmp2, tmp3, tmp4, NULL);
 	
 	g_free (tmp);
@@ -2487,7 +2094,9 @@ src_surroundings_logic ()
     if (message)
 	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
 	
-/*    fprintf (stderr, "\n%s", message); */
+#ifdef SRCTRL_DEBUG
+    fprintf (stderr, "\n%s", message); 
+#endif
     g_free (message);
     g_free (message1);
     g_free (message2);
@@ -2573,7 +2182,7 @@ src_overview ()
 	sprintf (cnt, "%d", role_cnt->cnt);
 	tmp2 = src_xml_make_part ("TEXT", src_speech_get_voice ("childcount"), cnt);
 	tmp3 = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), (gchar *)src_pres_get_role_name_for_speech (role_cnt->role));
-	tmp4 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), role_cnt->cnt == 1 ? _("item") : _("items"));
+	tmp4 = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), ngettext ("item", "items", role_cnt->cnt == 1));
 	message6 = g_strconcat (message6, tmp2, tmp3, tmp4, NULL);
 	
 	g_free (tmp);
@@ -2610,9 +2219,6 @@ src_overview ()
     return TRUE;
 }
 
-static gint flat_review_x = 0, flat_review_y = 0;
-/*static SRRectangle clip_rectangle = {0,0,300,300};*/
-static gint n_lines = 0;
 
 static gchar*
 src_present_line_for_speech (SRWAccLine *line)
@@ -2647,12 +2253,8 @@ src_present_line_for_speech (SRWAccLine *line)
 	tmp = spout;
 	spout = src_xml_format ("TEXT", src_speech_get_voice ("text"), spout);
 	g_free (tmp);
-/*	src_speech_send_chunk (spout, SRC_SPEECH_PRIORITY_IDLE);*/
-	/* fprintf (stderr, "\n%s", spout); */
     }
     
-/*    g_free (spout);*/
-    
     return (spout && spout[0]) ? spout : NULL;
 }
 
@@ -2687,7 +2289,6 @@ src_present_line_to_braille_or_brlmon (SRWAccLine *line)
 		};
 		break;
 	    case SRW_FILL_CELL:
-		break; /* FIXME this lien is here to see something on braille*/
 		{
 		    gchar *tmp;
 		    tmp = add;
@@ -2732,7 +2333,7 @@ src_present_line_to_braille_or_brlmon (SRWAccLine *line)
 	gchar offset[10];
 	tmp = brlout;
 	sprintf (offset, "%d", src_braille_offset);
-	brlout = g_strconcat ("<BRLOUT language=\"us\" bstyle=\"8\" clear=\"true\">",
+	brlout = g_strconcat ("<BRLOUT language=\"", src_braille_translation_table, "\" bstyle=\"8\" clear=\"true\">",
 				"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">",
 				brlout, "</BRLDISP>",
 				"</BRLOUT>",
@@ -2742,7 +2343,9 @@ src_present_line_to_braille_or_brlmon (SRWAccLine *line)
 	    src_braille_send (brlout);
 	if (src_use_braille_monitor)
 	    src_brlmon_send (brlout);
-	/* fprintf (stderr, "\n%s", brlout); */
+#ifdef SRCTRL_DEBUG
+    fprintf (stderr, "\n%s", brlout); 
+#endif
     }
     g_free (brlout);
     
@@ -2754,6 +2357,7 @@ static gboolean
 src_flat_review_line (gint index)
 {
     SRWAccLine *line = NULL;
+
     int y1 = 0, y2 = 0;
 
     sru_assert (1 <= index && index <= n_lines);
@@ -2766,7 +2370,22 @@ src_flat_review_line (gint index)
 			    "abs");
     if ((!line || line->is_empty) && src_use_speech)
     {
-	src_speech_say_message (_("empty line"));
+	if (line->is_empty == 1)
+	{
+	    gchar *message;
+	    message = src_xml_make_part ("TEXT", src_speech_get_voice ("empty"), _("empty line"));
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    g_free (message);
+	}
+	else
+	{
+	    gchar *aux_str = NULL, *message;
+	    aux_str = g_strdup_printf (ngettext ("%d empty line", "%d empty lines", line->is_empty), line->is_empty );
+	    message = src_xml_make_part ("TEXT", src_speech_get_voice ("empty"), aux_str);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    g_free (message);
+	    g_free (aux_str);
+	}
     }
     else if (line && src_use_speech)
     {
@@ -2779,8 +2398,24 @@ src_flat_review_line (gint index)
     
     if ((!line || line->is_empty) && (src_use_braille || src_use_braille_monitor))
     {
-	src_braille_show (_("[empty line]"));
-	src_brlmon_show (_("[empty line]"));
+	if (line->is_empty == 1)
+	{
+	    if (src_use_braille)
+		src_braille_show (_("[empty line]"));
+	    if (src_use_braille_monitor)
+		src_brlmon_show (_("[empty line]"));
+	}
+	else
+	{
+	    gchar *aux_str = NULL;
+	    aux_str = g_strdup_printf (ngettext ("[%d empty line]", "[%d empty lines]", line->is_empty), line->is_empty );
+	    if (src_use_braille)
+		src_braille_show (aux_str);
+	    if (src_use_braille_monitor)
+		src_brlmon_show (aux_str);
+	    g_free (aux_str);
+	}
+
     }
     else if (line && (src_use_braille || src_use_braille_monitor))
 	src_present_line_to_braille_or_brlmon (line);
@@ -2837,8 +2472,11 @@ src_flat_review ()
 			  &clip_rectangle,
 			  0);
 	sro_release_reference (window);
-	n_lines = screen_review_init (&clip_rectangle, screen_review_flag);
-	crt_line = 0;	
+	n_lines = screen_review_init (&clip_rectangle, 
+				      src_crt_sro,
+				      screen_review_flag);
+					      
+	crt_line = screen_review_get_focused_line ();	
 	flat_review_x = clip_rectangle.x;
 	src_flat_review_next_line();
     }
@@ -2852,20 +2490,6 @@ src_flat_review ()
 }
 
 
-#if 0
-/* just for test */
-static gboolean
-src_from_rect()
-{
-    GArray *array;
-    SRRectangle rect = {5, 5, 100, 100};
-
-    array = srr_collection_get_from_rect (&rect);
-    g_array_free (array, TRUE);
-    return TRUE;
-}
-#endif
-
 static gboolean
 get_hierarchy_from_sro (SRObject *obj, gchar **hierarchy)
 {
@@ -3011,7 +2635,7 @@ src_hierarchy_flat ()
 			  0);
 	sro_release_reference (window);
 	
-	n_lines = cnt = screen_review_init (&clip_rectangle, SRW_ALIGNF_ALL);
+	n_lines = cnt = screen_review_init (&clip_rectangle, src_crt_sro, SRW_ALIGNF_ALL);
 	flat_review_x = clip_rectangle.x;
     }
     spout = g_strdup ("");
@@ -3289,7 +2913,6 @@ static struct
 
 	{"attributes at caret", 	src_caret_attributes	 	},
 
-/*	{"flat review", 		src_from_rect			},*/
 	{"window hierarchy",		src_hierarchy_logic		},
 	{"window hierarchy flat",	src_hierarchy_flat		},
 	{"detailed informations",	src_detailed_informations	},
@@ -3393,12 +3016,15 @@ src_cmd_process (gchar *cmd)
     SRCSampleFunction function;
 
     sru_assert (cmd);
-    /* fprintf (stderr, "\nStart process %s", cmd); */
+#ifdef SRCTRL_DEBUG
+    fprintf (stderr, "\nStart process %s", cmd); 
+#endif
     function = g_hash_table_lookup (src_cmd_function_hash, cmd);
     if (function)
 	function ();
-    
-    /* fprintf (stderr, "\nEnd process %s", cmd); */
+#ifdef SRCTRL_DEBUG    
+    fprintf (stderr, "\nEnd process %s", cmd); 
+#endif
     return TRUE;
 }
 
@@ -3556,7 +3182,7 @@ src_key_cmds_mapped_add_from_list (gchar *entry, GSList *keys)
 	
 	if (!crt->data || !((gchar*)crt->data)[0])
 	{
-	    sru_warning (_("gconf \"%s\" entry has invalid value on %d-th position"),
+	    sru_warning (_("\"%s\" entry has invalid value at position %d."),
 				    entry, g_slist_position (keys, crt)); 
 	    continue;
 	}
@@ -3577,7 +3203,7 @@ src_key_cmds_mapped_add_from_list (gchar *entry, GSList *keys)
 	{
 	    if (!crt2->data || !((gchar*)crt2->data)[0])
 	    {
-		sru_warning (_("key \"%s\" has an invalid value on %d-th position"),
+		sru_warning (_("key \"%s\" has an invalid value at position %d."),
 				    (gchar*)crt->data, g_slist_position (functions, crt2));
 		continue;
 	    }
@@ -3613,7 +3239,7 @@ src_key_cmds_hash_init ()
     list = NULL;
     if (!srconf_get_data_with_default (SRC_KEY_PAD_LIST , CFGT_LIST,
 					&list, NULL, SRC_KEY_PAD_SECTION))
-	sru_warning (_("cannot get key pad comands mapping from gconf"));
+	sru_warning (_("Can not get key pad commands mapping from gconf."));
 
     if (list)
 	src_key_cmds_mapped_add_from_list (SRC_KEY_PAD_SECTION "/"
@@ -3624,7 +3250,7 @@ src_key_cmds_hash_init ()
     list = NULL;
     if (!srconf_get_data_with_default (SRC_BRAILLE_KEY_LIST , CFGT_LIST,
 			    &list, NULL, SRC_BRAILLE_KEY_SECTION))
-        sru_warning (_("cannot get braille display comands mapping from gconf"));
+        sru_warning (_("Can not get braille display commands mapping from gconf."));
     
     if (list)
 	src_key_cmds_mapped_add_from_list (SRC_BRAILLE_KEY_SECTION "/"
@@ -3635,7 +3261,7 @@ src_key_cmds_hash_init ()
     list = NULL;
     if (!srconf_get_data_with_default (SRC_USER_DEF_LIST , CFGT_LIST,
 			    &list, NULL, SRC_USER_DEF_SECTION))
-	sru_warning (_("cannot get user defined comands mapping from gconf"));
+	sru_warning (_("Can not get user defined commands mapping from gconf."));
 	
     if (list)
 	src_key_cmds_mapped_add_from_list (SRC_USER_DEF_SECTION "/"
@@ -3696,3 +3322,216 @@ src_ctrl_terminate ()
     return TRUE;
 }
 
+SRObject*
+src_ctrl_flat_get_sro_at_index (gint index)
+{
+    SRWAccLine *line = NULL;
+    SRWAccCell *cell;
+    int y1 = 0, y2 = 0;
+
+    if (1 > crt_line || crt_line > n_lines)
+	return NULL;
+    line = screen_review_get_line (crt_line, &y1, &y2);
+    if (!line)
+	return NULL;
+    if (index < 0 || index >= line->srw_acc_line->len)
+	return NULL;
+    cell = g_array_index (line->srw_acc_line, SRWAccCell *, index);
+    if (!cell)
+	return NULL;
+    sro_add_reference (cell->source);
+    return cell->source;
+}
+
+static void
+src_ctrl_position_sensor_action_flat (gint index)
+{
+    SRObject *sro;
+    gchar *role;
+    if (!src_use_speech)
+	return;
+    sro = src_ctrl_flat_get_sro_at_index (index);
+    if (sro && sro_get_role_name (sro, &role, SR_INDEX_CONTAINER))
+    {
+	gchar *message;
+	message = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), role);
+	if (message)
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	g_free (message);
+	SR_freeString (role);
+    }
+    else
+	src_speech_say_message (_("no role available"));	
+    sro_release_reference (sro);
+}
+
+static void
+src_ctrl_position_sensor_action_logic (gint index)
+{
+    if (!src_crt_sro)
+	return;
+	
+    src_text_index = index - 4 >= 0 ? index - 4 : 0;
+
+    switch (src_braille_position_sensor)
+    {
+	case 0:
+	    break;
+	case 1: 
+	    src_cmd_queue_add ("mouse goto current");
+	    src_cmd_queue_process ();
+	    break;
+	case 2:
+	    src_cmd_queue_add ("mouse goto current");
+	    src_cmd_queue_add ("mouse left click");
+	    src_cmd_queue_process ();
+	    break;
+	case 3:
+	    src_cmd_queue_add ("mouse goto current");
+	    src_cmd_queue_add ("mouse right click");
+	    src_cmd_queue_process ();
+	    break;
+	case 4:
+	    src_cmd_queue_add ("mouse goto current");
+/*
+	    src_cmd_queue_add ("mouse right click");
+	    src_cmd_queue_add ("mouse left click");
+*/
+	    src_cmd_queue_process ();
+	    break;
+	case 5:
+	    src_cmd_queue_add ("move caret");
+	    src_cmd_queue_process ();
+	    break;
+	case 6:
+	    src_cmd_queue_add ("show x coordinate");
+	    src_cmd_queue_process ();
+	    break;
+	case 7:
+	    src_cmd_queue_add ("show y coordinate");
+	    src_cmd_queue_process ();
+	    break;
+	case 8:
+	    src_cmd_queue_add ("font style");
+	    src_cmd_queue_process ();
+	    break;
+	case 9:
+	    src_cmd_queue_add ("font name");
+	    src_cmd_queue_process ();
+	    break;
+	case 10:
+	    src_cmd_queue_add ("font size");
+	    src_cmd_queue_process ();
+	    break;
+	case 11:
+	    sru_message ("sensor index 10 not implemented");
+	    break;
+	case 12:
+	    src_cmd_queue_add ("text attributes");
+	    src_cmd_queue_process ();
+	    break;
+	
+	default:
+	    sru_assert_not_reached ();
+	    break;
+    }	
+}
+
+#define src_ctrl_optical_sensor_action_flat src_ctrl_position_sensor_action_flat
+
+static void
+src_ctrl_optical_sensor_action_logic (gint index)
+{
+    if (!src_crt_sro)
+	return;
+
+    src_text_index = index - 4 >= 0 ? index - 4 : 0;
+
+    switch (src_braille_optical_sensor)
+    {
+	case 0:
+	    break;
+	case 1: 
+	    src_cmd_queue_add ("mouse goto current");
+	    src_cmd_queue_process ();
+	    break;
+	case 2:
+	    src_cmd_queue_add ("mouse goto current");
+	    src_cmd_queue_add ("mouse left click");
+	    src_cmd_queue_process ();
+	    break;
+	case 3:
+	    src_cmd_queue_add ("mouse goto current");
+	    src_cmd_queue_add ("mouse right click");
+	    src_cmd_queue_process ();
+	    break;
+	case 4:
+	    src_cmd_queue_add ("mouse goto current");
+/*
+	    src_cmd_queue_add ("mouse right click");
+	    src_cmd_queue_add ("mouse left click");
+*/
+	    src_cmd_queue_process ();
+	    break;
+	case 5:
+	    src_cmd_queue_add ("move caret");
+	    src_cmd_queue_process ();
+	    break;
+	case 6:
+	    src_cmd_queue_add ("show x coordinate");
+	    src_cmd_queue_process ();
+	    break;
+	case 7:
+	    src_cmd_queue_add ("show y coordinate");
+	    src_cmd_queue_process ();
+	    break;
+	case 8:
+	    src_cmd_queue_add ("font style");
+	    src_cmd_queue_process ();
+	    break;
+	case 9:
+	    src_cmd_queue_add ("font name");
+	    src_cmd_queue_process ();
+	    break;
+	case 10:
+	    src_cmd_queue_add ("font size");
+	    src_cmd_queue_process ();
+	    break;
+	case 11:
+	    sru_message ("sensor index 10 not implemented");
+	    break;
+	case 12:
+	    src_cmd_queue_add ("text attributes");
+	    src_cmd_queue_process ();
+	    break;
+	default:
+	    sru_assert_not_reached ();
+	    break;
+    }	
+}
+
+gboolean
+src_ctrl_position_sensor_action (gint index)
+{
+    if (src_tracking_mode == SRC_MODE_FOCUS_TRACKING)
+	src_ctrl_position_sensor_action_logic (index);
+    else if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
+	src_ctrl_position_sensor_action_flat (index);
+    else
+	sru_assert_not_reached ();
+
+    return TRUE;
+}
+
+gboolean
+src_ctrl_optical_sensor_action (gint index)
+{
+    if (src_tracking_mode == SRC_MODE_FOCUS_TRACKING)
+	src_ctrl_optical_sensor_action_logic (index);
+    else if (src_tracking_mode == SRC_MODE_FLAT_REVIEW)
+	src_ctrl_optical_sensor_action_flat (index);
+    else
+	sru_assert_not_reached ();
+
+    return TRUE;
+}
