@@ -29,7 +29,7 @@
 #include "libsrconf.h"
 #include "srctrl.h"
 
-
+#undef SRCORE_PANNING_DEBUG
 
 typedef struct
 {
@@ -108,14 +108,15 @@ gboolean
 src_magnifier_automatic_panning (gpointer data)
 {
     static gpointer	last = NULL;
-    static SRPoint 	point;  
+    static SRPoint 	point, start_point;  
     static gint reason;
+    static SRRectangle	old_location_line;
 
-    gchar *reason_;
     SRRectangle 	location_obj, location_line, location_char;
-    gchar 		*magout = NULL;
-    SRObject 		*obj    = (SRObject *)data;
-    int 		step_x  = 5;/*TBR*/
+    SRObject 		*obj    = (SRObject *)data;    
+    gchar 		*reason_, *magout = NULL;
+    gboolean		stop_panning = FALSE;
+    int 		step_x  = 2;/*TBR*/
 
     sro_add_reference (obj);
 
@@ -169,36 +170,38 @@ src_magnifier_automatic_panning (gpointer data)
 	{
 	    point.x = location_char.x;
 	    point.y = location_char.y;
+	    start_point = point;
 	}
 	else
 	{
 	    point.x = location_obj.x;
 	    point.y = location_obj.y;
-	    if (location_line.width < 0)
+	    start_point = point;
+	    
+/*	    if (location_line.width < 0)
 	    {
 		point.x += location_obj.width;
 		point.y += location_obj.height;
 	    }
-	}
-
+*/	}
     }
-
-    if (point.y > location_obj.y + location_obj.height)
+/*    if (point.y > location_obj.y + location_obj.height)
     {
 	sro_release_reference (obj);
-	return FALSE;
+	stop_panning = TRUE;
     }
-
+*/    
     if (!sro_text_get_text_location_from_point (obj, 
 						&point, 
 						SR_COORD_TYPE_SCREEN,
 						SR_TEXT_BOUNDARY_LINE, 
 						&location_line,
 						SR_INDEX_CONTAINER))
-	{
-	    sro_release_reference (obj);
-	    return FALSE;
-	}
+    {
+        sro_release_reference (obj);
+        return FALSE;
+    }
+    old_location_line = location_line;
 
     if (location_line.width > 0)
     {
@@ -227,6 +230,18 @@ src_magnifier_automatic_panning (gpointer data)
 	{
 	    point.x = location_obj.x;
 	    point.y = location_line.y + location_line.height;
+	    if (!sro_text_get_text_location_from_point (obj, 
+						&point, 
+						SR_COORD_TYPE_SCREEN,
+						SR_TEXT_BOUNDARY_LINE, 
+						&location_line,
+						SR_INDEX_CONTAINER))
+	    {
+		sro_release_reference (obj);
+		return FALSE;
+	    }
+	    if (location_line.y == old_location_line.y)
+		stop_panning = TRUE;
 	}
     }
     else
@@ -237,17 +252,32 @@ src_magnifier_automatic_panning (gpointer data)
 	{
 	    point.x = location_obj.x + location_obj.width;
 	    point.y = location_line.y + location_line.height;
+	    if (!sro_text_get_text_location_from_point (obj, 
+						&point, 
+						SR_COORD_TYPE_SCREEN,
+						SR_TEXT_BOUNDARY_LINE, 
+						&location_line,
+						SR_INDEX_CONTAINER))
+	    {
+		sro_release_reference (obj);
+		return FALSE;
+	    }
+	    if (location_line.y == old_location_line.y)
+		stop_panning = TRUE;
 	}
     }
-/*    
+    
+#ifdef SRCORE_PANNING_DEBUG
     fprintf (stderr,"\n !!!!!!PANNING is running %d %d %d %d  %d %d",
 		    location_line.x,
 		    location_line.y,
-		    location_line.width ,
-		    location_line.height,
-		    point..x,
+		    location_line.x + location_line.width ,
+		    location_line.y + location_line.height,
+		    point.x,
 		    point.y);
-*/
+#endif
+    if (stop_panning)
+	point = start_point;
     magout = g_strdup_printf ("<MAGOUT><ZOOMER ID =\"%s\" tracking=\"%s\" AlignmentX=\"0\" AlignementY=\"0\" ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\"></ZOOMER></MAGOUT>",
 				"generic_zoomer",
 				"panning",
@@ -256,6 +286,8 @@ src_magnifier_automatic_panning (gpointer data)
     src_magnifier_send (magout);
     g_free (magout);
     sro_release_reference (obj);
+    if (stop_panning)
+	return FALSE;
     return TRUE;
 }
 
@@ -281,19 +313,10 @@ src_magnifier_panning (SRObject *obj)
 
     if (panning)
     {
-/*
-	fprintf (stderr, "\n%xp %xp Start panning", 
-		    (unsigned int)old_panning, 
-		    (unsigned int)obj);
-*/
-	panning_handle = gtk_timeout_add (300,
+	panning_handle = gtk_timeout_add (100,
 					src_magnifier_automatic_panning,
 					old_panning);
     }
-/*
-    else
-	fprintf (stderr, "\nNO panning\n");
-*/
     return TRUE;
 }
 
@@ -301,12 +324,13 @@ src_magnifier_panning (SRObject *obj)
 gboolean
 src_magnifier_start_panning (SRObject *obj)
 {
-    SRObjectRoles role;
+/*    SRObjectRoles role;*/
 
     sru_assert (obj);
     
-    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-    if (role == SR_ROLE_TEXT)
+/*    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+    if (role == SR_ROLE_TEXT)*/
+    if (sro_is_text (obj, SR_INDEX_CONTAINER) )
     {
 	gchar *reason_;
 	if (sro_get_reason (obj, &reason_))
@@ -766,6 +790,14 @@ src_magnifier_load_values ()
 				       (gpointer)&default_bool_val, 
 				       MAGNIFIER_CONFIG_PATH)
 	) return NULL;
+	
+    default_bool_val = DEFAULT_MAGNIFIER_PANNING;
+    if (!srconf_get_data_with_default ( MAGNIFIER_PANNING,
+				        CFGT_BOOL, 
+					(gpointer)&panning, 
+					(gpointer)&default_bool_val, 
+					MAGNIFIER_CONFIG_PATH)
+	) return NULL; 	
 
     /*_________________________</ZOOMER  PARAMETERS>__________________________*/
     /*_________________________</ZOOMER  PROPERTIES>__________________________*/
