@@ -208,6 +208,10 @@ src_present_real_sro (SRObject *obj)
 			    g_free (src_last_key_echo);
 			    src_last_key_echo = NULL;
 			}
+			src_last_key_echo = diff;
+			src_cmd_queue_add ("kbd key");
+			src_cmd_queue_process ();
+			src_last_key_echo = NULL;
 		    }
 		    else
 		    {
@@ -1737,7 +1741,7 @@ src_kb_key_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_key_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_MESSAGE);
     g_free (message); 
     
     return TRUE;    
@@ -1802,9 +1806,9 @@ src_kb_cursor_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_cursor_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_SYSTEM);
     g_free (message); 
-    
+
     return TRUE;    
 }
 
@@ -2399,25 +2403,25 @@ src_get_defaults ()
 		    CFGT_INT, &src_braille_optical_sensor, &default_sensor);
     }
     
-    if (!src_use_braille)
+    if ((gint)src_use_braille == -1)
     {
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille, &src_use_braille);
     }
     
-    if (!src_use_braille_monitor)
+    if ((gint)src_use_braille_monitor == -1)
     {
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_MONITOR_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille_monitor, &src_use_braille_monitor);
     }
     
-    if (!src_use_speech)
+    if ((gint)src_use_speech == -1)
     {
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, 
 					    &src_use_speech, &src_use_speech);
     }
 
-    if (!src_use_magnifier)
+    if ((gint)src_use_magnifier == -1)
     {
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_MAGNIF_ACTIVE, CFGT_BOOL, 
 					    &src_use_magnifier, &src_use_magnifier);
@@ -2464,6 +2468,7 @@ src_init()
     SRLClient client;
     gchar *message = _("Welcome to Gnopernicus");
     gboolean brl_error = FALSE;
+    gboolean initialize_finished = FALSE;
 
     /* initialize SR Utils */
     sru_init();
@@ -2534,7 +2539,11 @@ src_init()
     src_last_cursor_echo 	= NULL;
     if (brl_error)
 	src_say_message (_("braille device can not be initialized"));
-    
+
+    /* src_init finished */
+    SET_SRCORE_CONFIG_DATA(SRCORE_INITIALIZATION_END, CFGT_BOOL, &initialize_finished);
+    initialize_finished = !initialize_finished;
+    SET_SRCORE_CONFIG_DATA(SRCORE_INITIALIZATION_END, CFGT_BOOL, &initialize_finished);    
 }
 
 static void
@@ -2620,21 +2629,111 @@ gint
 main (gint argc, 
       gchar **argv)
 {
-    gboolean braille 		= FALSE;
-    gboolean speech 		= FALSE;
-    gboolean magnifier 		= FALSE;
-    gboolean braille_monitor 	= FALSE;
-    
+    gboolean enable_braille 	= FALSE;
+    gboolean disable_braille 	= FALSE;
+    gboolean enable_speech 	= FALSE;
+    gboolean disable_speech 	= FALSE;
+    gboolean enable_magnifier	= FALSE;
+    gboolean disable_magnifier	= FALSE;
+    gboolean enable_braille_monitor 	= FALSE;
+    gboolean disable_braille_monitor 	= FALSE;    
 
     struct poptOption poptopt[] = 
     {		
-	{"braille",	  'b', 	POPT_ARG_NONE,  &braille,	'b', "Launch in execution with braille", NULL},
-	{"speech",	  's', 	POPT_ARG_NONE,  &speech,	's', "Launch in execution with speech", NULL},
-	{"magnifier",	  'm', 	POPT_ARG_NONE,  &magnifier,	'm', "Launch in execution with magnifier",  NULL},
-	{"braille-monitor", 'r', POPT_ARG_NONE, &braille_monitor, 'r', "Launch in execution with braille monitor", NULL},
-	{"braille-device",'e',  POPT_ARG_STRING, &src_braille_device,  'e', "Braille Device", 		"device_name"},
-	{"braille-port",  'p',  POPT_ARG_INT, 	&src_braille_port,'p', "Serial port (ttyS)", 	"serial_port_no"},
-	{NULL, 		0,0, NULL, 0}
+	{
+	 "enable-braille",	
+	 'b', 	
+	 POPT_ARG_NONE, 
+	 &enable_braille,		
+	 0, 
+	 "enable braille service", 
+	 NULL
+	},
+	{
+	 "disable-braille",	
+	 'B', 	
+	 POPT_ARG_NONE, 
+	 &disable_braille,		
+	 0, 
+	 "enable braille service", 
+	 NULL
+	},
+	{
+	 "enable-speech",	
+	 's', 	
+	 POPT_ARG_NONE, 
+	 &enable_speech,		
+	 0, 
+	 "enable speech service", 
+	 NULL
+	},
+	{
+	 "disable-speech",	
+	 'S', 	
+	 POPT_ARG_NONE, 
+	 &disable_speech,		
+	 0, 
+	 "disable speech service", 
+	 NULL
+	},
+	{
+	 "enable-magnifier",	
+	 'm', 	
+	 POPT_ARG_NONE, 
+	 &enable_magnifier,		
+	 0, 
+	 "enable magnifier service",  
+	 NULL
+	},
+	{
+	 "disable-magnifier",	
+	 'M', 	
+	 POPT_ARG_NONE, 
+	 &disable_magnifier,		
+	 0, 
+	 "disable magnifier service",  
+	 NULL
+	},
+
+	{
+	 "enable-braille-monitor", 
+	 'o', 
+	 POPT_ARG_NONE, 
+	 &enable_braille_monitor, 	
+	 0, 
+	 "enable braille monitor service", 
+	 NULL
+	},
+	{
+	 "disable-braille-monitor", 
+	 'O', 
+	 POPT_ARG_NONE, 
+	 &disable_braille_monitor, 	
+	 0, 
+	 "disable braille monitor service", 
+	 NULL
+	},
+	{
+	 "braille-port", 
+	 'p',   
+	 POPT_ARG_INT,  
+	 &src_braille_port, 	
+	 0, 
+	 "Serial port (ttyS)", 
+	 "ttyS[1..4]"
+	},
+	{
+	 "braille-device", 
+	 'e', 
+	 POPT_ARG_STRING, 
+	 &src_braille_device, 	
+	 0, 
+	 "Braille Device", 
+	 "DEVICE NAME"
+	},
+	{
+	 NULL, 0,     0, NULL, 0
+	}
     };
     
 #ifdef ENABLE_NLS
@@ -2664,10 +2763,17 @@ main (gint argc,
 	src_braille_port = DEFAULT_BRAILLE_PORT_NO;
     }
 
-    src_use_braille 		= braille;
-    src_use_speech 		= speech;
-    src_use_magnifier 		= magnifier;
-    src_use_braille_monitor 	= braille_monitor;
+    src_use_braille = (disable_braille ? FALSE :
+			(enable_braille ? TRUE : -1));
+
+    src_use_braille_monitor = (disable_braille_monitor ? FALSE :
+			      (enable_braille_monitor ? TRUE : -1));
+    
+    src_use_speech = (disable_speech ? FALSE :
+		     (enable_speech ? TRUE : -1));
+
+    src_use_magnifier = (disable_magnifier ? FALSE :
+			(enable_magnifier  ? TRUE : -1));
 
     src_init ();
 
