@@ -44,9 +44,9 @@ typedef enum
 typedef enum
 {
     SRC_SPEECH_PUNCTUATION_IGNORE,
-    SRC_SPEECH_PUNCTUATION_SAME,
-    SRC_SPEECH_PUNCTUATION_TRANSLATE,
-    SRC_SPEECH_PUNCTUATION_BOTH
+    SRC_SPEECH_PUNCTUATION_SOME,
+    SRC_SPEECH_PUNCTUATION_MOST,
+    SRC_SPEECH_PUNCTUATION_ALL
 }SRCSpeechPunctuationType;
 
 typedef enum
