@@ -26,11 +26,6 @@
 #include <glib-object.h>
 #include "SRMessages.h"
 
-/*
-#ifdef SR_DEBUG
-#define SRL_DEBUG
-#endif
-*/
 /*
 #define SRL_DEBUG
 */
@@ -426,6 +421,7 @@ gboolean sro_get_shortcut	(SRObject *obj, gchar **shortcut, SRLong index);
 gboolean sro_get_accelerator	(SRObject *obj, gchar **accelerator, SRLong index);
 gboolean sro_get_objs_for_relation(SRObject *obj, SRRelation type,
 					SRObject ***targets, SRLong index);
+gboolean sro_manages_descendants(SRObject *obj);
 gboolean sro_get_index_in_group (SRObject *obj, SRLong *index, SRLong index_obj);
 
 gboolean sro_action_get_count 	 	(SRObject *obj, SRLong *count, SRLong index);
